@@ -152,8 +152,7 @@ void SaveChannelAdmin(
 	channel->session().api().request(MTPchannels_EditAdmin(
 		channel->inputChannel,
 		user->inputUser,
-		MTP_chatAdminRights(MTP_flags(
-			MTPDchatAdminRights::Flags::from_raw(uint32(newRights.flags)))),
+		AdminRightsToMTP(newRights),
 		MTP_string(rank)
 	)).done([=](const MTPUpdates &result) {
 		channel->session().api().applyUpdates(result);
@@ -461,6 +460,7 @@ void ParticipantsAdditionalData::setExternal(
 		_adminRights.erase(user);
 		_adminCanEdit.erase(user);
 		_adminPromotedBy.erase(user);
+		_adminRanks.erase(user);
 		_admins.erase(user);
 	}
 	_restrictedRights.erase(participant);
@@ -538,6 +538,7 @@ void ParticipantsAdditionalData::fillFromChannel(
 			_adminRights.erase(user);
 			_adminCanEdit.erase(user);
 			_adminPromotedBy.erase(user);
+			_adminRanks.erase(user);
 			_restrictedRights.emplace(user, restricted->second.rights);
 		}
 	}
@@ -689,7 +690,7 @@ UserData *ParticipantsAdditionalData::applyAdmin(
 	const auto user = _peer->owner().userLoaded(data.userId());
 	if (!user) {
 		return nullptr;
-	} else if (const auto chat = _peer->asChat()) {
+	} else if (_peer->isChat()) {
 		// This can come from saveAdmin callback.
 		_admins.emplace(user);
 		return user;
@@ -733,7 +734,7 @@ UserData *ParticipantsAdditionalData::applyRegular(UserId userId) {
 	const auto user = _peer->owner().userLoaded(userId);
 	if (!user) {
 		return nullptr;
-	} else if (const auto chat = _peer->asChat()) {
+	} else if (_peer->isChat()) {
 		// This can come from saveAdmin or saveRestricted callback.
 		_admins.erase(user);
 		return user;
@@ -743,6 +744,7 @@ UserData *ParticipantsAdditionalData::applyRegular(UserId userId) {
 	_adminRights.erase(user);
 	_adminCanEdit.erase(user);
 	_adminPromotedBy.erase(user);
+	_adminRanks.erase(user);
 	_restrictedRights.erase(user);
 	_kicked.erase(user);
 	_restrictedBy.erase(user);
@@ -761,6 +763,7 @@ PeerData *ParticipantsAdditionalData::applyBanned(
 		_adminRights.erase(user);
 		_adminCanEdit.erase(user);
 		_adminPromotedBy.erase(user);
+		_adminRanks.erase(user);
 	}
 	if (data.isKicked()) {
 		_kicked.emplace(participant);
@@ -913,7 +916,7 @@ void ParticipantsBoxController::setupListChangeViewers() {
 				return;
 			}
 		}
-		if (const auto row = delegate()->peerListFindRow(user->id.value)) {
+		if (delegate()->peerListFindRow(user->id.value)) {
 			delegate()->peerListPartitionRows([&](const PeerListRow &row) {
 				return (row.peer() == user);
 			});
@@ -1270,6 +1273,33 @@ void ParticipantsBoxController::prepare() {
 	} else {
 		rebuild();
 	}
+
+	_peer->session().changes().chatAdminChanges(
+	) | rpl::start_with_next([=](const Data::ChatAdminChange &update) {
+		if (update.peer != _peer) {
+			return;
+		}
+		const auto user = update.user;
+		const auto rights = ChatAdminRightsInfo(update.rights);
+		const auto rank = update.rank;
+		_additional.applyAdminLocally(user, rights, rank);
+		if (!_additional.isCreator(user) || !user->isSelf()) {
+			if (!rights.flags) {
+				if (_role == Role::Admins) {
+					removeRow(user);
+				}
+			} else {
+				if (_role == Role::Admins) {
+					prependRow(user);
+				} else if (_role == Role::Kicked
+					|| _role == Role::Restricted) {
+					removeRow(user);
+				}
+			}
+		}
+		recomputeTypeFor(user);
+		refreshRows();
+	}, lifetime());
 }
 
 void ParticipantsBoxController::unload() {
@@ -1800,23 +1830,8 @@ void ParticipantsBoxController::editAdminDone(
 	if (_editParticipantBox) {
 		_editParticipantBox->closeBox();
 	}
-
-	_additional.applyAdminLocally(user, rights, rank);
-	if (!_additional.isCreator(user) || !user->isSelf()) {
-		if (!rights.flags) {
-			if (_role == Role::Admins) {
-				removeRow(user);
-			}
-		} else {
-			if (_role == Role::Admins) {
-				prependRow(user);
-			} else if (_role == Role::Kicked || _role == Role::Restricted) {
-				removeRow(user);
-			}
-		}
-	}
-	recomputeTypeFor(user);
-	refreshRows();
+	const auto flags = rights.flags;
+	user->session().changes().chatAdminChanged(_peer, user, flags, rank);
 }
 
 void ParticipantsBoxController::showRestricted(not_null<UserData*> user) {
