@@ -37,6 +37,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "passport/passport_form_controller.h"
 #include "ui/text/text_utilities.h"
 #include "ui/toast/toast.h"
+#include "ui/vertical_list.h"
 #include "data/components/credits.h"
 #include "data/data_birthday.h"
 #include "data/data_channel.h"
@@ -302,6 +303,42 @@ void ShowLanguagesBox(Window::SessionController *controller) {
 	Guard = LanguageBox::Show(controller);
 }
 
+void ShowPhonePrivacyBox(Window::SessionController *controller) {
+	static auto Guard = base::binary_guard();
+	auto guard = base::binary_guard();
+
+	using Privacy = Api::UserPrivacy;
+	const auto key = Privacy::Key::PhoneNumber;
+	controller->session().api().userPrivacy().reload(key);
+
+	const auto weak = base::make_weak(controller);
+	auto shared = std::make_shared<base::binary_guard>(
+		guard.make_guard());
+	auto lifetime = std::make_shared<rpl::lifetime>();
+	controller->session().api().userPrivacy().value(
+		key
+	) | rpl::take(
+		1
+	) | rpl::start_with_next([=](const Privacy::Rule &value) mutable {
+		using namespace ::Settings;
+		const auto show = shared->alive();
+		if (lifetime) {
+			base::take(lifetime)->destroy();
+		}
+		if (show) {
+			if (const auto controller = weak.get()) {
+				controller->show(Box<EditPrivacyBox>(
+					controller,
+					std::make_unique<PhoneNumberPrivacyController>(
+						controller),
+					value));
+			}
+		}
+	}, *lifetime);
+
+	Guard = std::move(guard);
+}
+
 bool SetLanguage(
 		Window::SessionController *controller,
 		const Match &match,
@@ -483,6 +520,8 @@ bool ShowWallPaper(
 			result |= ChatAdminRight::AddAdmins;
 		} else if (element == u"manage_video_chats"_q) {
 			result |= ChatAdminRight::ManageCall;
+		} else if (element == u"manage_direct_messages"_q) {
+			result |= ChatAdminRight::ManageDirect;
 		} else if (element == u"anonymous"_q) {
 			result |= ChatAdminRight::Anonymous;
 		} else if (element == u"manage_chat"_q) {
@@ -718,6 +757,9 @@ bool ResolveSettings(
 		if (section == u"language"_q) {
 			ShowLanguagesBox(controller);
 			return {};
+		} else if (section == u"phone_privacy"_q) {
+			ShowPhonePrivacyBox(controller);
+			return {};
 		} else if (section == u"devices"_q) {
 			return ::Settings::Sessions::Id();
 		} else if (section == u"folders"_q) {
@@ -807,6 +849,7 @@ bool OpenMediaTimestamp(
 				document,
 				context,
 				context ? context->topicRootId() : MsgId(0),
+				context ? context->sublistPeerId() : PeerId(0),
 				false,
 				time));
 		} else if (document->isSong() || document->isVoiceMessage()) {
@@ -872,6 +915,8 @@ bool ShowEditBirthday(
 		const QVariant &context) {
 	if (!controller) {
 		return false;
+	} else if (controller->showFrozenError()) {
+		return true;
 	}
 	const auto user = controller->session().user();
 	const auto save = [=](Data::Birthday result) {
@@ -895,10 +940,41 @@ bool ShowEditBirthday(
 				: (u"Error: "_q + error.type()));
 		})).handleFloodErrors().send();
 	};
-	controller->show(Box(
-		Ui::EditBirthdayBox,
-		user->birthday(),
-		save));
+	if (match->captured(1).isEmpty()) {
+		controller->show(Box(Ui::EditBirthdayBox, user->birthday(), save));
+	} else {
+		controller->show(Box([=](not_null<Ui::GenericBox*> box) {
+			Ui::EditBirthdayBox(box, user->birthday(), save);
+
+			const auto container = box->verticalLayout();
+			const auto session = &user->session();
+			const auto key = Api::UserPrivacy::Key::Birthday;
+			session->api().userPrivacy().reload(key);
+			auto isExactlyContacts = session->api().userPrivacy().value(
+				key
+			) | rpl::map([=](const Api::UserPrivacy::Rule &value) {
+				return (value.option == Api::UserPrivacy::Option::Contacts)
+					&& value.always.peers.empty()
+					&& !value.always.premiums
+					&& value.never.peers.empty();
+			}) | rpl::distinct_until_changed();
+			Ui::AddSkip(container);
+			const auto link = u"internal:edit_privacy_birthday:from_box"_q;
+			Ui::AddDividerText(container, rpl::conditional(
+				std::move(isExactlyContacts),
+				tr::lng_settings_birthday_contacts(
+					lt_link,
+					tr::lng_settings_birthday_contacts_link(
+					) | Ui::Text::ToLink(link),
+					Ui::Text::WithEntities),
+				tr::lng_settings_birthday_about(
+					lt_link,
+					tr::lng_settings_birthday_about_link(
+					) | Ui::Text::ToLink(link),
+					Ui::Text::WithEntities)));
+		}));
+
+	}
 	return true;
 }
 
@@ -909,11 +985,29 @@ bool ShowEditBirthdayPrivacy(
 	if (!controller) {
 		return false;
 	}
+	const auto isFromBox = !match->captured(1).isEmpty();
 	auto syncLifetime = controller->session().api().userPrivacy().value(
 		Api::UserPrivacy::Key::Birthday
 	) | rpl::take(
 		1
 	) | rpl::start_with_next([=](const Api::UserPrivacy::Rule &value) {
+		if (isFromBox) {
+			using namespace ::Settings;
+			class Controller final : public BirthdayPrivacyController {
+				object_ptr<Ui::RpWidget> setupAboveWidget(
+					not_null<Window::SessionController*> controller,
+					not_null<QWidget*> parent,
+					rpl::producer<Option> optionValue,
+					not_null<QWidget*> outerContainer) override {
+					return { nullptr };
+				}
+			};
+			controller->show(Box<EditPrivacyBox>(
+				controller,
+				std::make_unique<Controller>(),
+				value));
+			return;
+		}
 		controller->show(Box<EditPrivacyBox>(
 			controller,
 			std::make_unique<::Settings::BirthdayPrivacyController>(),
@@ -928,6 +1022,8 @@ bool ShowEditPersonalChannel(
 		const QVariant &context) {
 	if (!controller) {
 		return false;
+	} else if (controller->showFrozenError()) {
+		return true;
 	}
 
 	auto listController = std::make_unique<PersonalChannelController>(
@@ -1405,6 +1501,35 @@ bool ResolveUniqueGift(
 	return true;
 }
 
+bool ResolveConferenceCall(
+		Window::SessionController *controller,
+		const Match &match,
+		const QVariant &context) {
+	if (!controller) {
+		return false;
+	}
+	const auto slug = match->captured(1);
+	if (slug.isEmpty()) {
+		return false;
+	}
+	const auto myContext = context.value<ClickHandlerContext>();
+	controller->window().activate();
+	controller->resolveConferenceCall(match->captured(1), myContext.itemId);
+	return true;
+}
+
+bool ResolveStarsSettings(
+		Window::SessionController *controller,
+		const Match &match,
+		const QVariant &context) {
+	if (!controller) {
+		return false;
+	}
+	controller->showSettings(::Settings::CreditsId());
+	controller->window().activate();
+	return true;
+}
+
 } // namespace
 
 const std::vector<LocalUrlHandler> &LocalUrlHandlers() {
@@ -1462,7 +1587,7 @@ const std::vector<LocalUrlHandler> &LocalUrlHandlers() {
 			ResolvePrivatePost
 		},
 		{
-			u"^settings(/language|/devices|/folders|/privacy|/themes|/change_number|/auto_delete|/information|/edit_profile)?$"_q,
+			u"^settings(/language|/devices|/folders|/privacy|/themes|/change_number|/auto_delete|/information|/edit_profile|/phone_privacy)?$"_q,
 			ResolveSettings
 		},
 		{
@@ -1501,6 +1626,14 @@ const std::vector<LocalUrlHandler> &LocalUrlHandlers() {
 			u"^nft/?\\?slug=([a-zA-Z0-9\\.\\_\\-]+)(&|$)"_q,
 			ResolveUniqueGift
 		},
+		{
+			u"^call/?\\?slug=([a-zA-Z0-9\\.\\_\\-]+)(&|$)"_q,
+			ResolveConferenceCall
+		},
+		{
+			u"^stars/?(^\\?.*)?(#|$)"_q,
+			ResolveStarsSettings
+		},
 		{
 			u"^([^\\?]+)(\\?|#|$)"_q,
 			HandleUnknown
@@ -1532,11 +1665,11 @@ const std::vector<LocalUrlHandler> &InternalUrlHandlers() {
 			ShowSearchTagsPromo
 		},
 		{
-			u"^edit_birthday$"_q,
+			u"^edit_birthday(.*)$"_q,
 			ShowEditBirthday,
 		},
 		{
-			u"^edit_privacy_birthday$"_q,
+			u"^edit_privacy_birthday(.*)$"_q,
 			ShowEditBirthdayPrivacy,
 		},
 		{
@@ -1661,6 +1794,9 @@ QString TryConvertUrlToLocal(QString url) {
 		} else if (const auto nftMatch = regex_match(u"^nft/([a-zA-Z0-9\\.\\_\\-]+)(\\?|$)"_q, query, matchOptions)) {
 			const auto slug = nftMatch->captured(1);
 			return u"tg://nft?slug="_q + slug;
+		} else if (const auto callMatch = regex_match(u"^call/([a-zA-Z0-9\\.\\_\\-]+)(\\?|$)"_q, query, matchOptions)) {
+			const auto slug = callMatch->captured(1);
+			return u"tg://call?slug="_q + slug;
 		} else if (const auto privateMatch = regex_match(u"^"
 			"c/(\\-?\\d+)"
 			"("
@@ -1787,7 +1923,7 @@ void ResolveAndShowUniqueGift(
 		session->data().processUsers(data.vusers());
 		if (const auto gift = Api::FromTL(session, data.vgift())) {
 			using namespace ::Settings;
-			show->show(Box(GlobalStarGiftBox, show, *gift, st));
+			show->show(Box(GlobalStarGiftBox, show, *gift, PeerId(), st));
 		}
 	}).fail([=](const MTP::Error &error) {
 		clear();
