@@ -10,6 +10,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "api/api_text_entities.h"
 #include "data/business/data_shortcut_messages.h"
 #include "data/components/scheduled_messages.h"
+#include "data/data_saved_sublist.h"
 #include "data/data_session.h"
 #include "data/data_channel.h"
 #include "data/data_chat.h"
@@ -60,6 +61,14 @@ MTPInputReplyTo ReplyToForMTP(
 			&& (to->history() != history || to->id != replyingToTopicId))
 			? to->topicRootId()
 			: replyingToTopicId;
+		const auto possibleMonoforumPeerId = (to && to->sublistPeerId())
+			? to->sublistPeerId()
+			: replyTo.monoforumPeerId
+			? replyTo.monoforumPeerId
+			: history->session().user()->id;
+		const auto replyToMonoforumPeerId = history->peer->amMonoforumAdmin()
+			? possibleMonoforumPeerId
+			: PeerId();
 		const auto external = replyTo.messageId
 			&& (replyTo.messageId.peer != history->peer->id
 				|| replyingToTopicId != replyToTopicId);
@@ -74,6 +83,9 @@ MTPInputReplyTo ReplyToForMTP(
 				| (replyTo.quote.text.isEmpty()
 					? Flag()
 					: (Flag::f_quote_text | Flag::f_quote_offset))
+				| (replyToMonoforumPeerId
+					? Flag::f_monoforum_peer_id
+					: Flag())
 				| (quoteEntities.v.isEmpty()
 					? Flag()
 					: Flag::f_quote_entities)),
@@ -84,7 +96,16 @@ MTPInputReplyTo ReplyToForMTP(
 				: MTPInputPeer()),
 			MTP_string(replyTo.quote.text),
 			quoteEntities,
-			MTP_int(replyTo.quoteOffset));
+			MTP_int(replyTo.quoteOffset),
+			(replyToMonoforumPeerId
+				? history->owner().peer(replyToMonoforumPeerId)->input
+				: MTPInputPeer()));
+	} else if (history->peer->amMonoforumAdmin()
+		&& replyTo.monoforumPeerId) {
+		const auto replyToMonoforumPeer = replyTo.monoforumPeerId
+			? history->owner().peer(replyTo.monoforumPeerId)
+			: history->session().user();
+		return MTP_inputReplyToMonoForum(replyToMonoforumPeer->input);
 	}
 	return MTPInputReplyTo();
 }
@@ -475,10 +496,29 @@ void Histories::changeDialogUnreadMark(
 	using Flag = MTPmessages_MarkDialogUnread::Flag;
 	session().api().request(MTPmessages_MarkDialogUnread(
 		MTP_flags(unread ? Flag::f_unread : Flag(0)),
+		MTPInputPeer(), // parent_peer
 		MTP_inputDialogPeer(history->peer->input)
 	)).send();
 }
 
+void Histories::changeSublistUnreadMark(
+		not_null<Data::SavedSublist*> sublist,
+		bool unread) {
+	const auto parent = sublist->parentChat();
+	if (!parent) {
+		return;
+	}
+	sublist->setUnreadMark(unread);
+
+	using Flag = MTPmessages_MarkDialogUnread::Flag;
+	session().api().request(MTPmessages_MarkDialogUnread(
+		MTP_flags(Flag::f_parent_peer
+			| (unread ? Flag::f_unread : Flag(0))),
+		parent->input,
+		MTP_inputDialogPeer(sublist->sublistPeer()->input)
+	)).send();
+}
+
 void Histories::requestFakeChatListMessage(
 		not_null<History*> history) {
 	if (_fakeChatListRequests.contains(history)) {
@@ -671,6 +711,7 @@ void Histories::sendReadRequest(not_null<History*> history, State &state) {
 			} else {
 				Assert(!state->sentReadTill || state->sentReadTill > tillId);
 			}
+			history->validateMonoforumUnread(tillId);
 			sendReadRequests();
 			finish();
 		};
@@ -772,7 +813,7 @@ void Histories::deleteAllMessages(
 				channel->inputChannel,
 				MTP_int(deleteTillId)
 			)).done(finish).fail(finish).send();
-		} else if (revoke && chat && chat->amCreator()) {
+		} else if (!justClear && revoke && chat && chat->amCreator()) {
 			return session().api().request(MTPmessages_DeleteChat(
 				chat->inputChat
 			)).done(finish).fail([=](const MTP::Error &error) {
@@ -1054,13 +1095,12 @@ int Histories::sendPreparedMessage(
 		_creatingTopicRequests.emplace(id);
 		return id;
 	}
-	const auto realReplyTo = FullReplyTo{
-		.messageId = convertTopicReplyToId(history, replyTo.messageId),
-		.quote = replyTo.quote,
-		.storyId = replyTo.storyId,
-		.topicRootId = convertTopicReplyToId(history, replyTo.topicRootId),
-		.quoteOffset = replyTo.quoteOffset,
+	auto realReplyTo = replyTo;
+	const auto topicReplyToId = [&](const auto &id) {
+		return convertTopicReplyToId(history, id);
 	};
+	realReplyTo.messageId = topicReplyToId(replyTo.messageId);
+	realReplyTo.topicRootId = topicReplyToId(replyTo.topicRootId);
 	return v::match(message(history, realReplyTo), [&](const auto &request) {
 		const auto type = RequestType::Send;
 		return sendRequest(history, type, [=](Fn<void()> finish) {
