@@ -10,6 +10,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "scheme.h"
 #include "base/optional.h"
 #include "base/variant.h"
+#include "core/credits_amount.h"
 #include "data/data_peer_id.h"
 
 #include <QtCore/QSize>
@@ -44,6 +45,39 @@ inline auto NumberToString(Type value, int length = 0, char filler = '0')
 		filler).replace(',', '.');
 }
 
+struct TextPart {
+	enum class Type {
+		Text,
+		Unknown,
+		Mention,
+		Hashtag,
+		BotCommand,
+		Url,
+		Email,
+		Bold,
+		Italic,
+		Code,
+		Pre,
+		TextUrl,
+		MentionName,
+		Phone,
+		Cashtag,
+		Underline,
+		Strike,
+		Blockquote,
+		BankCard,
+		Spoiler,
+		CustomEmoji,
+	};
+	Type type = Type::Text;
+	Utf8String text;
+	Utf8String additional;
+
+	[[nodiscard]] static Utf8String UnavailableEmoji() {
+		return "(unavailable)";
+	}
+};
+
 struct UserpicsInfo {
 	int count = 0;
 };
@@ -198,19 +232,31 @@ struct PaidMedia {
 
 struct Poll {
 	struct Answer {
-		Utf8String text;
+		std::vector<TextPart> text;
 		QByteArray option;
 		int votes = 0;
 		bool my = false;
 	};
 
 	uint64 id = 0;
-	Utf8String question;
+	std::vector<TextPart> question;
 	std::vector<Answer> answers;
 	int totalVotes = 0;
 	bool closed = false;
 };
 
+struct TodoListItem {
+	std::vector<TextPart> text;
+	int id = 0;
+};
+
+struct TodoList {
+	bool othersCanAppend = false;
+	bool othersCanComplete = false;
+	std::vector<TextPart> title;
+	std::vector<TodoListItem> items;
+};
+
 struct GiveawayStart {
 	std::vector<QString> countries;
 	std::vector<ChannelId> channels;
@@ -370,6 +416,7 @@ struct Media {
 		Game,
 		Invoice,
 		Poll,
+		TodoList,
 		GiveawayStart,
 		GiveawayResults,
 		PaidMedia,
@@ -404,39 +451,6 @@ Media ParseMedia(
 	const QString &folder,
 	TimeId date);
 
-struct TextPart {
-	enum class Type {
-		Text,
-		Unknown,
-		Mention,
-		Hashtag,
-		BotCommand,
-		Url,
-		Email,
-		Bold,
-		Italic,
-		Code,
-		Pre,
-		TextUrl,
-		MentionName,
-		Phone,
-		Cashtag,
-		Underline,
-		Strike,
-		Blockquote,
-		BankCard,
-		Spoiler,
-		CustomEmoji,
-	};
-	Type type = Type::Text;
-	Utf8String text;
-	Utf8String additional;
-
-	[[nodiscard]] static Utf8String UnavailableEmoji() {
-		return "(unavailable)";
-	}
-};
-
 struct ActionChatCreate {
 	Utf8String title;
 	std::vector<UserId> userIds;
@@ -497,15 +511,19 @@ struct ActionPaymentSent {
 };
 
 struct ActionPhoneCall {
-	enum class DiscardReason {
+	enum class State {
 		Unknown,
 		Missed,
 		Disconnect,
 		Hangup,
 		Busy,
-		AllowGroupCall,
+		MigrateConferenceCall,
+		Invitation,
+		Active,
 	};
-	DiscardReason discardReason = DiscardReason::Unknown;
+
+	uint64 conferenceId = 0;
+	State state = State::Unknown;
 	int duration = 0;
 };
 
@@ -641,9 +659,9 @@ struct ActionPaymentRefunded {
 	Utf8String transactionId;
 };
 
-struct ActionGiftStars {
+struct ActionGiftCredits {
 	Utf8String cost;
-	int credits = 0;
+	CreditsAmount amount;
 };
 
 struct ActionPrizeStars {
@@ -662,6 +680,41 @@ struct ActionStarGift {
 	bool limited = false;
 };
 
+struct ActionPaidMessagesRefunded {
+	int messages = 0;
+	int64 stars = 0;
+};
+
+struct ActionPaidMessagesPrice {
+	int stars = 0;
+	bool broadcastAllowed = false;
+};
+
+struct ActionTodoCompletions {
+	std::vector<int> completed;
+	std::vector<int> incompleted;
+};
+
+struct ActionTodoAppendTasks {
+	std::vector<TodoListItem> items;
+};
+
+struct ActionSuggestedPostApproval {
+	Utf8String rejectComment;
+	TimeId scheduleDate = 0;
+	CreditsAmount price;
+	bool rejected = false;
+	bool balanceTooLow = false;
+};
+
+struct ActionSuggestedPostSuccess {
+	CreditsAmount price;
+};
+
+struct ActionSuggestedPostRefund {
+	bool payerInitiated = false;
+};
+
 struct ServiceAction {
 	std::variant<
 		v::null_t,
@@ -705,9 +758,16 @@ struct ServiceAction {
 		ActionGiveawayResults,
 		ActionBoostApply,
 		ActionPaymentRefunded,
-		ActionGiftStars,
+		ActionGiftCredits,
 		ActionPrizeStars,
-		ActionStarGift> content;
+		ActionStarGift,
+		ActionPaidMessagesRefunded,
+		ActionPaidMessagesPrice,
+		ActionTodoCompletions,
+		ActionTodoAppendTasks,
+		ActionSuggestedPostApproval,
+		ActionSuggestedPostSuccess,
+		ActionSuggestedPostRefund> content;
 };
 
 ServiceAction ParseServiceAction(
