@@ -131,6 +131,7 @@ MTPMessage PrepareLogMessage(const MTPMessage &message, TimeId newDate) {
 		const auto reply = PrepareLogReply(data.vreply_to());
 		const auto removeFlags = Flag::f_out
 			| Flag::f_post
+			| Flag::f_saved_peer_id
 			| Flag::f_reactions_are_possible
 			| Flag::f_reactions
 			| Flag::f_ttl_period
@@ -140,6 +141,7 @@ MTPMessage PrepareLogMessage(const MTPMessage &message, TimeId newDate) {
 			data.vid(),
 			data.vfrom_id() ? *data.vfrom_id() : MTPPeer(),
 			data.vpeer_id(),
+			MTPPeer(), // saved_peer_id
 			reply.value_or(MTPMessageReplyHeader()),
 			MTP_int(newDate),
 			data.vaction(),
@@ -150,6 +152,7 @@ MTPMessage PrepareLogMessage(const MTPMessage &message, TimeId newDate) {
 		const auto reply = PrepareLogReply(data.vreply_to());
 		const auto removeFlags = Flag::f_out
 			| Flag::f_post
+			| Flag::f_saved_peer_id
 			| (reply ? Flag() : Flag::f_reply_to)
 			| Flag::f_replies
 			| Flag::f_edit_date
@@ -160,7 +163,8 @@ MTPMessage PrepareLogMessage(const MTPMessage &message, TimeId newDate) {
 			| Flag::f_restriction_reason
 			| Flag::f_ttl_period
 			| Flag::f_factcheck
-			| Flag::f_report_delivery_until_date;
+			| Flag::f_report_delivery_until_date
+			| Flag::f_suggested_post;
 		return MTP_message(
 			MTP_flags(data.vflags().v & ~removeFlags),
 			data.vid(),
@@ -192,7 +196,8 @@ MTPMessage PrepareLogMessage(const MTPMessage &message, TimeId newDate) {
 			MTP_long(data.veffect().value_or_empty()),
 			MTPFactCheck(),
 			MTPint(), // report_delivery_until_date
-			MTP_long(data.vpaid_message_stars().value_or_empty()));
+			MTP_long(data.vpaid_message_stars().value_or_empty()),
+			MTPSuggestedPost());
 	});
 }
 
@@ -282,6 +287,7 @@ TextWithEntities GenerateAdminChangeText(
 		{ Flag::ManageTopics, tr::lng_admin_log_admin_manage_topics },
 		{ Flag::PinMessages, tr::lng_admin_log_admin_pin_messages },
 		{ Flag::ManageCall, tr::lng_admin_log_admin_manage_calls },
+		{ Flag::ManageDirect, tr::lng_admin_log_admin_manage_direct },
 		{ Flag::AddAdmins, tr::lng_admin_log_admin_add_admins },
 		{ Flag::Anonymous, tr::lng_admin_log_admin_remain_anonymous },
 	};
@@ -831,6 +837,7 @@ void GenerateItems(
 	using LogChangeEmojiStatus = MTPDchannelAdminLogEventActionChangeEmojiStatus;
 	using LogToggleSignatureProfiles = MTPDchannelAdminLogEventActionToggleSignatureProfiles;
 	using LogParticipantSubExtend = MTPDchannelAdminLogEventActionParticipantSubExtend;
+	using LogToggleAutotranslation = MTPDchannelAdminLogEventActionToggleAutotranslation;
 
 	const auto session = &history->session();
 	const auto id = event.vid().v;
@@ -2115,6 +2122,18 @@ void GenerateItems(
 			participantPeerLink);
 	};
 
+	const auto createToggleAutotranslation = [&](const LogToggleAutotranslation &action) {
+		const auto enabled = mtpIsTrue(action.vnew_value());
+		const auto text = (enabled
+			? tr::lng_admin_log_autotranslate_enabled
+			: tr::lng_admin_log_autotranslate_disabled)(
+				tr::now,
+				lt_from,
+				fromLinkText,
+				Ui::Text::WithEntities);
+		addSimpleServiceMessage(text);
+	};
+
 	action.match(
 		createChangeTitle,
 		createChangeAbout,
@@ -2165,7 +2184,8 @@ void GenerateItems(
 		createChangeWallpaper,
 		createChangeEmojiStatus,
 		createToggleSignatureProfiles,
-		createParticipantSubExtend);
+		createParticipantSubExtend,
+		createToggleAutotranslation);
 }
 
 } // namespace AdminLog
