@@ -48,6 +48,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_file_click_handler.h"
 #include "data/data_session.h"
 #include "data/data_stories.h"
+#include "data/data_todo_list.h"
 #include "main/main_session.h"
 #include "window/window_session_controller.h"
 #include "api/api_bot.h"
@@ -70,6 +71,38 @@ base::options::toggle FastButtonsModeOption({
 	.description = "Trigger inline keyboard buttons by 1-9 keyboard keys.",
 });
 
+[[nodiscard]] TextWithEntities ComposeTodoTasksList(
+		int fullCount,
+		const std::vector<TextWithEntities> &names) {
+	const auto count = int(names.size());
+	if (!count) {
+		return tr::lng_action_todo_tasks_fallback(
+			tr::now,
+			lt_count,
+			fullCount,
+			Ui::Text::WithEntities);
+	} else if (count == 1) {
+		return names.front();
+	}
+	auto full = names.front();
+	for (auto i = 1; i != count - 1; ++i) {
+		full = tr::lng_action_todo_tasks_and_one(
+			tr::now,
+			lt_tasks,
+			full,
+			lt_task,
+			names[i],
+			Ui::Text::WithEntities);
+	}
+	return tr::lng_action_todo_tasks_and_last(
+		tr::now,
+		lt_tasks,
+		full,
+		lt_task,
+		names.back(),
+		Ui::Text::WithEntities);
+}
+
 } // namespace
 
 const char kOptionFastButtonsMode[] = "fast-buttons-mode";
@@ -196,7 +229,7 @@ bool HiddenSenderInfo::paintCustomUserpic(
 		image.isNull() ? nullptr : &image,
 		image.isNull() ? &emptyUserpic : nullptr,
 		size * style::DevicePixelRatio(),
-		false);
+		Ui::PeerUserpicShape::Circle);
 	p.drawImage(QRect(x, y, size, size), view.cached);
 	return valid;
 }
@@ -417,6 +450,13 @@ FullReplyTo ReplyToFromMTP(
 			};
 		}
 		return FullReplyTo();
+	}, [&](const MTPDinputReplyToMonoForum &data) {
+		const auto parsed = Data::PeerFromInputMTP(
+			&history->owner(),
+			data.vmonoforum_peer_id());
+		return FullReplyTo{
+			.monoforumPeerId = parsed ? parsed->id : PeerId(),
+		};
 	});
 }
 
@@ -719,17 +759,26 @@ ReplyKeyboard::ReplyKeyboard(
 			for (auto j = 0; j != rowSize; ++j) {
 				auto button = Button();
 				using Type = HistoryMessageMarkupButton::Type;
-				const auto isBuy = (row[j].type == Type::Buy);
 				static const auto RegExp = QRegularExpression("\\b"
 					+ Ui::kCreditsCurrency
 					+ "\\b");
-				const auto text = isBuy
+				const auto type = row[j].type;
+				const auto text = (type == Type::Buy)
 					? base::duplicate(row[j].text).replace(
 						RegExp,
 						QChar(0x2B50))
 					: row[j].text;
+				const auto withEmoji = [&](const style::IconEmoji &icon) {
+					return Ui::Text::IconEmoji(&icon).append(text);
+				};
 				const auto textWithEntities = [&] {
-					if (!isBuy) {
+					if (type == Type::SuggestAccept) {
+						return withEmoji(st::chatSuggestAcceptIcon);
+					} else if (type == Type::SuggestDecline) {
+						return withEmoji(st::chatSuggestDeclineIcon);
+					} else if (type == Type::SuggestChange) {
+						return withEmoji(st::chatSuggestChangeIcon);
+					} else if (type != Type::Buy) {
 						return TextWithEntities();
 					}
 					auto result = TextWithEntities();
@@ -745,7 +794,7 @@ ReplyKeyboard::ReplyKeyboard(
 						? TextWithEntities()
 						: result;
 				}();
-				button.type = row.at(j).type;
+				button.type = type;
 				button.link = std::make_shared<ReplyMarkupClickHandler>(
 					owner,
 					i,
@@ -1174,6 +1223,95 @@ bool HistoryMessageReplyMarkup::hiddenBy(Data::Media *media) const {
 	return false;
 }
 
+void HistoryMessageReplyMarkup::updateSuggestControls(
+		SuggestionActions actions) {
+	if (actions == SuggestionActions::AcceptAndDecline) {
+		data.flags |= ReplyMarkupFlag::SuggestionAccept;
+	} else {
+		data.flags &= ~ReplyMarkupFlag::SuggestionAccept;
+	}
+	if (actions == SuggestionActions::None) {
+		data.flags &= ~ReplyMarkupFlag::SuggestionDecline;
+	} else {
+		data.flags |= ReplyMarkupFlag::Inline
+			| ReplyMarkupFlag::SuggestionDecline;
+	}
+	using Type = HistoryMessageMarkupButton::Type;
+	const auto has = [&](Type type) {
+		return !data.rows.empty()
+			&& ranges::contains(
+				data.rows.back(),
+				type,
+				&HistoryMessageMarkupButton::type);
+	};
+	if (actions == SuggestionActions::AcceptAndDecline) {
+		//     ... rows ...
+		// [decline] | [accept]
+		//   [suggestchanges]
+		if (has(Type::SuggestChange)) {
+			// Nothing changed.
+		} else {
+			if (has(Type::SuggestDecline)) {
+				data.rows.pop_back();
+			}
+			data.rows.push_back({
+				{
+					Type::SuggestDecline,
+					tr::lng_suggest_action_decline(tr::now),
+				},
+				{
+					Type::SuggestAccept,
+					tr::lng_suggest_action_accept(tr::now),
+				},
+			});
+			data.rows.push_back({ {
+				Type::SuggestChange,
+				tr::lng_suggest_action_change(tr::now),
+			} });
+			data.flags |= ReplyMarkupFlag::SuggestionAccept
+				| ReplyMarkupFlag::SuggestionDecline;
+		}
+		if (data.rows.size() > 2) {
+			data.flags |= ReplyMarkupFlag::SuggestionSeparator;
+		} else {
+			data.flags &= ~ReplyMarkupFlag::SuggestionSeparator;
+		}
+	} else {
+		while (!data.rows.empty()) {
+			if (has(Type::SuggestChange) || has(Type::SuggestAccept)) {
+				data.rows.pop_back();
+			} else if (has(Type::SuggestDecline)
+				&& actions == SuggestionActions::None) {
+				data.rows.pop_back();
+			} else {
+				break;
+			}
+		}
+		data.flags &= ~ReplyMarkupFlag::SuggestionAccept;
+		if (actions == SuggestionActions::None) {
+			data.flags &= ~ReplyMarkupFlag::SuggestionDecline;
+			data.flags &= ~ReplyMarkupFlag::SuggestionSeparator;
+		} else {
+			if (!has(Type::SuggestDecline)) {
+				// ... rows ...
+				//  [decline]
+				data.rows.push_back({ {
+					Type::SuggestDecline,
+					tr::lng_suggest_action_decline(tr::now),
+				} });
+				data.flags |= ReplyMarkupFlag::SuggestionDecline;
+			}
+			if (data.rows.size() > 1) {
+				data.flags |= ReplyMarkupFlag::SuggestionSeparator;
+			} else {
+				data.flags &= ~ReplyMarkupFlag::SuggestionSeparator;
+			}
+		}
+	}
+
+	inlineKeyboard = nullptr;
+}
+
 HistoryMessageLogEntryOriginal::HistoryMessageLogEntryOriginal() = default;
 
 HistoryMessageLogEntryOriginal::HistoryMessageLogEntryOriginal(
@@ -1218,6 +1356,38 @@ MessageFactcheck FromMTP(
 	return result;
 }
 
+TextWithEntities ComposeTodoTasksList(
+		HistoryItem *itemWithList,
+		const std::vector<int> &ids) {
+	const auto media = itemWithList ? itemWithList->media() : nullptr;
+	const auto list = media ? media->todolist() : nullptr;
+	auto names = std::vector<TextWithEntities>();
+	if (list) {
+		names.reserve(ids.size());
+		for (const auto &id : ids) {
+			const auto i = ranges::find(list->items, id, &TodoListItem::id);
+			if (i == end(list->items)) {
+				names.clear();
+				break;
+			}
+			names.push_back(
+				TextWithEntities().append('"').append(i->text).append('"'));
+		}
+	}
+	return ComposeTodoTasksList(ids.size(), names);
+}
+
+TextWithEntities ComposeTodoTasksList(
+		not_null<HistoryServiceTodoAppendTasks*> append) {
+	auto names = std::vector<TextWithEntities>();
+	names.reserve(append->list.size());
+	for (const auto &task : append->list) {
+		names.push_back(
+			TextWithEntities().append('"').append(task.text).append('"'));
+	}
+	return ComposeTodoTasksList(names.size(), names);
+}
+
 HistoryDocumentCaptioned::HistoryDocumentCaptioned()
 : caption(st::msgFileMinWidth - st::msgPadding.left() - st::msgPadding.right()) {
 }
