@@ -15,6 +15,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "history/history.h"
 #include "history/history_drag_area.h"
 #include "history/history_item_helpers.h" // GetErrorForSending.
+#include "history/history_view_swipe_back_session.h"
 #include "menu/menu_send.h" // SendMenu::Type.
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/tooltip.h"
@@ -230,6 +231,8 @@ ScheduledWidget::ScheduledWidget(
 				_composeControls->editMessage(
 					fullId,
 					_inner->getSelectedTextRange(item));
+			} else if (media->todolist()) {
+				Window::PeerMenuEditTodoList(controller, item);
 			}
 		}
 	}, _inner->lifetime());
@@ -245,6 +248,7 @@ ScheduledWidget::ScheduledWidget(
 		_inner->setEmptyInfoWidget(std::move(emptyInfo));
 	}
 	setupComposeControls();
+	Window::SetupSwipeBackSection(this, _scroll, _inner);
 }
 
 ScheduledWidget::~ScheduledWidget() = default;
@@ -266,15 +270,22 @@ void ScheduledWidget::setupComposeControls() {
 					: tr::lng_forum_topic_closed(tr::now);
 			});
 			return rpl::combine(
+				session().frozenValue(),
 				session().changes().peerFlagsValue(
 					_history->peer,
 					Data::PeerUpdate::Flag::Rights),
 				Data::CanSendAnythingValue(_history->peer),
 				std::move(topicWriteRestrictions)
 			) | rpl::map([=](
+					const Main::FreezeInfo &info,
 					auto,
 					auto,
 					Data::SendError topicRestriction) {
+				if (info) {
+					return Controls::WriteRestriction{
+						.type = Controls::WriteRestrictionType::Frozen,
+					};
+				}
 				const auto allWithoutPolls = Data::AllSendRestrictions()
 					& ~ChatRestriction::SendPolls;
 				const auto canSendAnything = Data::CanSendAnyOf(
@@ -301,11 +312,17 @@ void ScheduledWidget::setupComposeControls() {
 		}()
 		: [&] {
 			return rpl::combine(
+				session().frozenValue(),
 				session().changes().peerFlagsValue(
 					_history->peer,
 					Data::PeerUpdate::Flag::Rights),
 				Data::CanSendAnythingValue(_history->peer)
-			) | rpl::map([=] {
+			) | rpl::map([=](const Main::FreezeInfo &info, auto, auto) {
+				if (info) {
+					return Controls::WriteRestriction{
+						.type = Controls::WriteRestrictionType::Frozen,
+					};
+				}
 				const auto allWithoutPolls = Data::AllSendRestrictions()
 					& ~ChatRestriction::SendPolls;
 				const auto canSendAnything = Data::CanSendAnyOf(
