@@ -29,6 +29,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "settings/settings_credits_graphics.h" // GiftedCreditsBox
 #include "settings/settings_premium.h" // Settings::ShowGiftPremium
 #include "ui/chat/chat_style.h"
+#include "ui/controls/ton_common.h" // kNanosInOne
 #include "ui/layers/generic_box.h"
 #include "ui/text/text_utilities.h"
 #include "window/window_session_controller.h"
@@ -47,7 +48,7 @@ PremiumGift::PremiumGift(
 PremiumGift::~PremiumGift() = default;
 
 int PremiumGift::top() {
-	return starGift()
+	return (starGift() || tonGift())
 		? st::msgServiceStarGiftStickerTop
 		: st::msgServiceGiftBoxStickerTop;
 }
@@ -57,7 +58,7 @@ int PremiumGift::width() {
 }
 
 QSize PremiumGift::size() {
-	return starGift()
+	return (starGift() || tonGift())
 		? QSize(
 			st::msgServiceStarGiftStickerSize,
 			st::msgServiceStarGiftStickerSize)
@@ -68,7 +69,13 @@ QSize PremiumGift::size() {
 
 TextWithEntities PremiumGift::title() {
 	using namespace Ui::Text;
-	if (starGift()) {
+	if (tonGift()) {
+		return tr::lng_gift_ton_amount(
+			tr::now,
+			lt_count_decimal,
+			CreditsAmount(0, _data.count, CreditsType::Ton).value(),
+			Ui::Text::WithEntities);
+	} else if (starGift()) {
 		const auto peer = _parent->history()->peer;
 		return peer->isSelf()
 			? tr::lng_action_gift_self_subtitle(tr::now, WithEntities)
@@ -114,7 +121,9 @@ TextWithEntities PremiumGift::title() {
 }
 
 TextWithEntities PremiumGift::subtitle() {
-	if (starGift()) {
+	if (tonGift()) {
+		return tr::lng_action_gift_got_ton(tr::now, Ui::Text::WithEntities);
+	} else if (starGift()) {
 		const auto toChannel = _data.channel
 			&& _parent->history()->peer->isServiceUser();
 		return !_data.message.empty()
@@ -237,11 +246,34 @@ rpl::producer<QString> PremiumGift::button() {
 		: tr::lng_prize_open();
 }
 
-bool PremiumGift::buttonMinistars() {
-	return true;
+std::optional<Ui::Premium::MiniStarsType> PremiumGift::buttonMinistars() {
+	return tonGift()
+		? Ui::Premium::MiniStarsType::SlowDiamondStars
+		: Ui::Premium::MiniStarsType::SlowStars;
 }
 
 ClickHandlerPtr PremiumGift::createViewLink() {
+	if (tonGift()) {
+		const auto lifetime = std::make_shared<rpl::lifetime>();
+		return std::make_shared<LambdaClickHandler>([=](ClickContext context) {
+			const auto my = context.other.value<ClickHandlerContext>();
+			const auto weak = my.sessionWindow;
+			if (const auto window = weak.get()) {
+				window->session().credits().tonLoad();
+				*lifetime = window->session().credits().tonLoadedValue(
+				) | rpl::filter([=] {
+					if (const auto window = weak.get()) {
+						return window->session().credits().tonLoaded();
+					}
+					return false;
+				}) | rpl::take(1) | rpl::start_with_next([=] {
+					if (const auto window = weak.get()) {
+						window->showSettings(Settings::CurrencyId());
+					}
+				});
+			}
+		});
+	}
 	if (auto link = OpenStarGiftLink(_parent->data())) {
 		return link;
 	}
@@ -376,6 +408,10 @@ bool PremiumGift::gift() const {
 	return _data.slug.isEmpty() || !_data.channel;
 }
 
+bool PremiumGift::tonGift() const {
+	return (_data.type == Data::GiftType::Ton);
+}
+
 bool PremiumGift::starGift() const {
 	return (_data.type == Data::GiftType::StarGift);
 }
@@ -397,6 +433,19 @@ int PremiumGift::credits() const {
 void PremiumGift::ensureStickerCreated() const {
 	if (_sticker) {
 		return;
+	} else if (tonGift()) {
+		const auto &session = _parent->history()->session();
+		auto &packs = session.giftBoxStickersPacks();
+		const auto count = _data.count / Ui::kNanosInOne;
+		if (const auto document = packs.tonLookup(count)) {
+			if (document->sticker()) {
+				const auto skipPremiumEffect = false;
+				_sticker.emplace(_parent, document, skipPremiumEffect, _parent);
+				_sticker->setStopOnLastFrame(true);
+				_sticker->initSize(st::msgServiceGiftBoxStickerSize);
+			}
+		}
+		return;
 	} else if (const auto document = _data.document) {
 		const auto sticker = document->sticker();
 		Assert(sticker != nullptr);
@@ -411,10 +460,10 @@ void PremiumGift::ensureStickerCreated() const {
 	const auto count = credits();
 	const auto months = count ? packs.monthsForStars(count) : _data.count;
 	if (const auto document = packs.lookup(months)) {
-		if (const auto sticker = document->sticker()) {
+		if (document->sticker()) {
 			const auto skipPremiumEffect = false;
 			_sticker.emplace(_parent, document, skipPremiumEffect, _parent);
-			_sticker->setPlayingOnce(true);
+			_sticker->setStopOnLastFrame(true);
 			_sticker->initSize(st::msgServiceGiftBoxStickerSize);
 		}
 	}
@@ -479,7 +528,8 @@ ClickHandlerPtr OpenStarGiftLink(not_null<HistoryItem*> item) {
 						Settings::SavedStarGiftBox,
 						window,
 						owner,
-						*parsed));
+						*parsed,
+						nullptr));
 				}
 			}
 		}).fail([=](const MTP::Error &error) {
