@@ -7,20 +7,25 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "info/peer_gifts/info_peer_gifts_common.h"
 
+#include "boxes/send_credits_box.h" // SetButtonMarkedLabel
 #include "boxes/star_gift_box.h"
 #include "boxes/sticker_set_box.h"
 #include "chat_helpers/stickers_gift_box_pack.h"
 #include "chat_helpers/stickers_lottie.h"
 #include "core/ui_integration.h"
 #include "data/stickers/data_custom_emoji.h"
+#include "data/data_credits.h" // CreditsHistoryEntry
 #include "data/data_document.h"
 #include "data/data_document_media.h"
 #include "data/data_session.h"
 #include "history/view/media/history_view_sticker_player.h"
 #include "lang/lang_keys.h"
 #include "main/main_session.h"
+#include "settings/settings_credits_graphics.h"
+#include "ui/layers/generic_box.h"
 #include "ui/text/format_values.h"
 #include "ui/text/text_utilities.h"
+#include "ui/widgets/buttons.h"
 #include "ui/dynamic_image.h"
 #include "ui/dynamic_thumbnails.h"
 #include "ui/effects/premium_graphics.h"
@@ -50,10 +55,14 @@ std::strong_ordering operator<=>(const GiftBadge &a, const GiftBadge &b) {
 	if (result3 != std::strong_ordering::equal) {
 		return result3;
 	}
-	const auto result4 = (a.fg.rgb() <=> b.fg.rgb());
+	const auto result4 = (a.border.rgb() <=> b.border.rgb());
 	if (result4 != std::strong_ordering::equal) {
 		return result4;
 	}
+	const auto result5 = (a.fg.rgb() <=> b.fg.rgb());
+	if (result5 != std::strong_ordering::equal) {
+		return result5;
+	}
 	return a.gradient <=> b.gradient;
 }
 
@@ -75,14 +84,22 @@ void GiftButton::unsubscribe() {
 }
 
 void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
-	if (_descriptor == descriptor) {
+	const auto unique = v::is<GiftTypeStars>(descriptor)
+		? v::get<GiftTypeStars>(descriptor).info.unique.get()
+		: nullptr;
+	const auto resalePrice = unique ? unique->starsForResale : 0;
+	if (_descriptor == descriptor && _resalePrice == resalePrice) {
 		return;
 	}
 	auto player = base::take(_player);
-	const auto starsType = Ui::Premium::MiniStars::Type::SlowStars;
+	const auto starsType = Ui::Premium::MiniStarsType::SlowStars;
 	_mediaLifetime.destroy();
-	_descriptor = descriptor;
 	unsubscribe();
+
+	_descriptor = descriptor;
+	_resalePrice = resalePrice;
+	const auto resale = (_resalePrice > 0);
+	_small = (mode != Mode::Full);
 	v::match(descriptor, [&](const GiftTypePremium &data) {
 		const auto months = data.months;
 		_text = Ui::Text::String(st::giftBoxGiftHeight / 4);
@@ -120,7 +137,6 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 			{ 1., st::windowActiveTextFg->c },
 		});
 	}, [&](const GiftTypeStars &data) {
-		const auto unique = data.info.unique.get();
 		const auto soldOut = data.info.limitedCount
 			&& !data.userpic
 			&& !data.info.limitedLeft;
@@ -129,15 +145,28 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 			: data.from
 			? Ui::MakeUserpicThumbnail(data.from)
 			: Ui::MakeHiddenAuthorThumbnail();
-		if (mode == Mode::Minimal) {
+		if (_small && !resale) {
 			_price = {};
 			_stars.reset();
 			return;
 		}
 		_price.setMarkedText(
 			st::semiboldTextStyle,
-			(unique
-				? tr::lng_gift_price_unique(tr::now, Ui::Text::WithEntities)
+			(data.resale
+				? (unique
+					? _delegate->monostar()
+					: _delegate->star()).append(' ').append(
+						Lang::FormatCountDecimal(unique
+							? unique->starsForResale
+							: data.info.starsResellMin)
+					).append(data.info.resellCount > 1 ? "+" : "")
+				: (_small && unique && unique->starsForResale)
+				? _delegate->monostar().append(' ').append(
+					Lang::FormatCountDecimal(unique->starsForResale))
+				: unique
+				? tr::lng_gift_transfer_button(
+					tr::now,
+					Ui::Text::WithEntities)
 				: _delegate->star().append(
 					' ' + Lang::FormatCountDecimal(data.info.stars))),
 			kMarkupTextOptions,
@@ -151,6 +180,9 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 				{ 0., anim::with_alpha(white, .3) },
 				{ 1., white },
 			});
+		} else if (data.resale) {
+			_stars->setColorOverride(
+				Ui::Premium::CreditsIconGradientStops());
 		} else if (soldOut) {
 			_stars.reset();
 		} else {
@@ -168,9 +200,8 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 	_uniquePatternEmoji = nullptr;
 	_uniquePatternCache.clear();
 
-	if (mode != Mode::Full) {
+	if (_small && !resale) {
 		_button = QRect();
-		_small = true;
 		return;
 	}
 	const auto buttonw = _price.maxWidth();
@@ -180,7 +211,9 @@ void GiftButton::setDescriptor(const GiftDescriptor &descriptor, Mode mode) {
 		QSize(buttonw, buttonh)
 	).marginsAdded(st::giftBoxButtonPadding);
 	const auto skipy = _delegate->buttonSize().height()
-		- (_byStars.isEmpty()
+		- (_small
+			? st::giftBoxButtonBottomSmall
+			: _byStars.isEmpty()
 			? st::giftBoxButtonBottom
 			: st::giftBoxButtonBottomByStars)
 		- inner.height();
@@ -240,6 +273,79 @@ void GiftButton::setGeometry(QRect inner, QMargins extend) {
 	AbstractButton::setGeometry(inner.marginsAdded(extend));
 }
 
+QMargins GiftButton::currentExtend() const {
+	const auto progress = _selectedAnimation.value(_selected ? 1. : 0.);
+	const auto added = anim::interpolate(0, st::giftBoxSelectSkip, progress);
+	return _extend + QMargins(added, added, added, added);
+}
+
+void GiftButton::toggleSelected(bool selected) {
+	if (_selected == selected) {
+		return;
+	}
+	const auto duration = st::defaultRoundCheckbox.duration;
+	_selected = selected;
+	_selectedAnimation.start([=] {
+		update();
+	}, selected ? 0. : 1., selected ? 1. : 0., duration, anim::easeOutCirc);
+}
+
+void GiftButton::paintBackground(QPainter &p, const QImage &background) {
+	const auto removed = currentExtend() - _extend;
+	const auto x = removed.left();
+	const auto y = removed.top();
+	const auto width = this->width() - x - removed.right();
+	const auto height = this->height() - y - removed.bottom();
+	const auto dpr = int(background.devicePixelRatio());
+	const auto bwidth = background.width() / dpr;
+	const auto bheight = background.height() / dpr;
+	const auto fillRow = [&](int yfrom, int ytill, int bfrom) {
+		const auto fill = [&](int xto, int wto, int xfrom, int wfrom = 0) {
+			const auto fheight = ytill - yfrom;
+			p.drawImage(
+				QRect(x + xto, y + yfrom, wto, fheight),
+				background,
+				QRect(
+					QPoint(xfrom, bfrom) * dpr,
+					QSize((wfrom ? wfrom : wto), fheight) * dpr));
+		};
+		if (width < bwidth) {
+			const auto xhalf = width / 2;
+			fill(0, xhalf, 0);
+			fill(xhalf, width - xhalf, bwidth - (width - xhalf));
+		} else if (width == bwidth) {
+			fill(0, width, 0);
+		} else {
+			const auto half = bwidth / (2 * dpr);
+			fill(0, half, 0);
+			fill(width - half, half, bwidth - half);
+			fill(half, width - 2 * half, half, 1);
+		}
+	};
+	if (height < bheight) {
+		fillRow(0, height / 2, 0);
+		fillRow(height / 2, height, bheight - (height - (height / 2)));
+	} else {
+		fillRow(0, height, 0);
+	}
+
+	auto hq = PainterHighQualityEnabler(p);
+	const auto progress = _selectedAnimation.value(_selected ? 1. : 0.);
+	if (progress < 0.01) {
+		return;
+	}
+	const auto pwidth = progress * st::defaultRoundCheckbox.width;
+	p.setPen(QPen(st::defaultRoundCheckbox.bgActive->c, pwidth));
+	p.setBrush(Qt::NoBrush);
+	const auto rounded = rect().marginsRemoved(_extend);
+	const auto phalf = pwidth / 2.;
+	const auto extended = QRectF(rounded).marginsRemoved(
+		{ phalf, phalf, phalf, phalf });
+	const auto xradius = removed.left() + st::giftBoxGiftRadius - phalf;
+	const auto yradius = removed.top() + st::giftBoxGiftRadius - phalf;
+	p.drawRoundedRect(extended, xradius, yradius);
+}
+
 void GiftButton::resizeEvent(QResizeEvent *e) {
 	if (!_button.isEmpty()) {
 		_button.moveLeft((width() - _button.width()) / 2);
@@ -267,9 +373,10 @@ void GiftButton::cacheUniqueBackground(
 		[[maybe_unused]] const auto preload = _uniquePatternEmoji->ready();
 	}
 	const auto outer = QRect(0, 0, width, height);
+	const auto extend = currentExtend();
 	const auto inner = outer.marginsRemoved(
-		_extend
-	).translated(-_extend.left(), -_extend.top());
+		extend
+	).translated(-extend.left(), -extend.top());
 	const auto ratio = style::DevicePixelRatio();
 	if (_uniqueBackgroundCache.size() != inner.size() * ratio) {
 		_uniqueBackgroundCache = QImage(
@@ -311,34 +418,18 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 	const auto unique = v::is<GiftTypeStars>(_descriptor)
 		? v::get<GiftTypeStars>(_descriptor).info.unique.get()
 		: nullptr;
+	const auto onsale = (unique && unique->starsForResale && _small);
 	const auto hidden = v::is<GiftTypeStars>(_descriptor)
 		&& v::get<GiftTypeStars>(_descriptor).hidden;;
-	const auto position = QPoint(_extend.left(), _extend.top());
+	const auto extend = currentExtend();
+	const auto position = QPoint(extend.left(), extend.top());
 	const auto background = _delegate->background();
-	const auto dpr = int(background.devicePixelRatio());
 	const auto width = this->width();
-	if (width * dpr <= background.width()) {
-		p.drawImage(0, 0, background);
-	} else {
-		const auto full = background.width();
-		const auto half = ((full / 2) / dpr) * dpr;
-		const auto height = background.height();
-		p.drawImage(
-			QRect(0, 0, half / dpr, height / dpr),
-			background,
-			QRect(0, 0, half, height));
-		p.drawImage(
-			QRect(width - (half / dpr), 0, half / dpr, height / dpr),
-			background,
-			QRect(full - half, 0, half, height));
-		p.drawImage(
-			QRect(half / dpr, 0, width - 2 * (half / dpr), height / dpr),
-			background,
-			QRect(half, 0, 1, height));
-	}
+	const auto dpr = int(background.devicePixelRatio());
+	paintBackground(p, background);
 	if (unique) {
 		cacheUniqueBackground(unique, width, background.height() / dpr);
-		p.drawImage(_extend.left(), _extend.top(), _uniqueBackgroundCache);
+		p.drawImage(extend.left(), extend.top(), _uniqueBackgroundCache);
 	}
 
 	if (_userpic) {
@@ -348,7 +439,7 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 		}
 		const auto image = _userpic->image(st::giftBoxUserpicSize);
 		const auto skip = st::giftBoxUserpicSkip;
-		p.drawImage(_extend.left() + skip, _extend.top() + skip, image);
+		p.drawImage(extend.left() + skip, extend.top() + skip, image);
 	}
 
 	auto frame = QImage();
@@ -401,7 +492,7 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 
 	auto hq = PainterHighQualityEnabler(p);
 	const auto premium = v::is<GiftTypePremium>(_descriptor);
-	const auto singlew = width - _extend.left() - _extend.right();
+	const auto singlew = width - extend.left() - extend.right();
 	const auto font = st::semiboldFont;
 	p.setFont(font);
 
@@ -420,10 +511,20 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 		}
 		return GiftBadge();
 	}, [&](const GiftTypeStars &data) {
-		if (const auto count = data.info.limitedCount) {
-			const auto soldOut = !data.userpic && !data.info.limitedLeft;
+		const auto count = data.info.limitedCount;
+		const auto pinned = data.pinned || data.pinnedSelection;
+		if (count || pinned) {
+			const auto soldOut = !pinned
+				&& !data.userpic
+				&& !data.info.limitedLeft;
 			return GiftBadge{
-				.text = (soldOut
+				.text = (onsale
+					? tr::lng_gift_stars_on_sale(tr::now)
+					: (unique && (data.resale || pinned))
+					? ('#' + QString::number(unique->number))
+					: data.resale
+					? tr::lng_gift_stars_resale(tr::now)
+					: soldOut
 					? tr::lng_gift_stars_sold_out(tr::now)
 					: (!data.userpic && !data.info.unique)
 					? tr::lng_gift_stars_limited(tr::now)
@@ -435,15 +536,28 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 						(((count % 1000) && (count < 10'000))
 							? Lang::FormatCountDecimal(count)
 							: Lang::FormatCountToShort(count).string))),
-				.bg1 = (unique
+				.bg1 = (onsale
+					? st::boxTextFgGood->c
+					: unique
 					? unique->backdrop.edgeColor
+					: data.resale
+					? st::boxTextFgGood->c
 					: soldOut
 					? st::attentionButtonFg->c
 					: st::windowActiveTextFg->c),
-				.bg2 = (unique
+				.bg2 = (onsale
+					? QColor(0, 0, 0, 0)
+					: unique
 					? unique->backdrop.patternColor
 					: QColor(0, 0, 0, 0)),
-				.fg = unique ? QColor(255, 255, 255) : st::windowBg->c,
+				.border = (onsale
+					? QColor(255, 255, 255)
+					: QColor(0, 0, 0, 0)),
+				.fg = (onsale
+					? st::windowBg->c
+					: unique
+					? QColor(255, 255, 255)
+					: st::windowBg->c),
 				.small = true,
 			};
 		}
@@ -452,7 +566,7 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 
 	if (badge) {
 		const auto rubberOut = st::lineWidth;
-		const auto inner = rect().marginsRemoved(_extend);
+		const auto inner = rect().marginsRemoved(extend);
 		p.setClipRect(inner.marginsAdded(
 			{ rubberOut, rubberOut, rubberOut, rubberOut }));
 
@@ -474,7 +588,7 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 			p.setPen(Qt::NoPen);
 			p.setBrush(unique->backdrop.patternColor);
 			const auto rect = QRect(
-				QPoint(_extend.left() + skip, _extend.top() + skip),
+				QPoint(extend.left() + skip, extend.top() + skip),
 				QSize(icon.width() + 2 * add, icon.height() + 2 * add));
 			p.drawEllipse(rect);
 			icon.paintInCenter(p, rect);
@@ -482,7 +596,9 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 	});
 
 	if (!_button.isEmpty()) {
-		p.setBrush(unique
+		p.setBrush(onsale
+			? QBrush(unique->backdrop.patternColor)
+			: unique
 			? QBrush(QColor(255, 255, 255, .2 * 255))
 			: premium
 			? st::lightButtonBgOver
@@ -490,11 +606,13 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 		p.setPen(Qt::NoPen);
 		if (!unique && !premium) {
 			p.setOpacity(0.12);
+		} else if (onsale) {
+			p.setOpacity(0.8);
 		}
 		const auto geometry = _button;
 		const auto radius = geometry.height() / 2.;
 		p.drawRoundedRect(geometry, radius, radius);
-		if (!premium) {
+		if (!premium || onsale) {
 			p.setOpacity(1.);
 		}
 		if (_stars) {
@@ -547,12 +665,10 @@ void GiftButton::paintEvent(QPaintEvent *e) {
 	}
 }
 
-Delegate::Delegate(
-	not_null<Window::SessionController*> window,
-	GiftButtonMode mode)
-: _window(window)
+Delegate::Delegate(not_null<Main::Session*> session, GiftButtonMode mode)
+: _session(session)
 , _hiddenMark(std::make_unique<StickerPremiumMark>(
-	&window->session(),
+	_session,
 	st::giftBoxHiddenMark,
 	RectPart::Center))
 , _mode(mode) {
@@ -563,18 +679,21 @@ Delegate::Delegate(Delegate &&other) = default;
 Delegate::~Delegate() = default;
 
 TextWithEntities Delegate::star() {
-	const auto owner = &_window->session().data();
-	return owner->customEmojiManager().creditsEmoji();
+	return _session->data().customEmojiManager().creditsEmoji();
+}
+
+TextWithEntities Delegate::monostar() {
+	return Ui::Text::IconEmoji(&st::starIconEmoji);
 }
 
 TextWithEntities Delegate::ministar() {
-	const auto owner = &_window->session().data();
+	const auto owner = &_session->data();
 	const auto top = st::giftBoxByStarsStarTop;
 	return owner->customEmojiManager().ministarEmoji({ 0, top, 0, 0 });
 }
 
 Ui::Text::MarkedContext Delegate::textContext() {
-	return Core::TextContext({ .session = &_window->session() });
+	return Core::TextContext({ .session = _session });
 }
 
 QSize Delegate::buttonSize() {
@@ -601,7 +720,7 @@ auto Delegate::buttonPatternEmoji(
 	not_null<Data::UniqueGift*> unique,
 	Fn<void()> repaint)
 -> std::unique_ptr<Ui::Text::CustomEmoji> {
-	return _window->session().data().customEmojiManager().create(
+	return _session->data().customEmojiManager().create(
 		unique->pattern.document,
 		repaint,
 		Data::CustomEmojiSizeTag::Large);
@@ -658,7 +777,7 @@ QImage Delegate::background() {
 
 rpl::producer<not_null<DocumentData*>> Delegate::sticker(
 		const GiftDescriptor &descriptor) {
-	return GiftStickerValue(&_window->session(), descriptor);
+	return GiftStickerValue(_session, descriptor);
 }
 
 not_null<StickerPremiumMark*> Delegate::hiddenMark() {
@@ -720,6 +839,7 @@ QImage ValidateRotatedBadge(const GiftBadge &badge, int added) {
 	const auto ratio = style::DevicePixelRatio();
 	const auto multiplier = ratio * 3;
 	const auto size = (twidth + font->height * 2);
+	const auto height = font->height + st::lineWidth;
 	const auto textpos = QPoint(size - skip, added);
 	auto image = QImage(
 		QSize(size, size) * multiplier,
@@ -750,12 +870,16 @@ QImage ValidateRotatedBadge(const GiftBadge &badge, int added) {
 	{
 		auto p = QPainter(&result);
 		auto hq = PainterHighQualityEnabler(p);
-		p.setPen(Qt::NoPen);
 
 		p.save();
 		p.translate(textpos);
 		p.rotate(45.);
-		const auto rect = QRect(-5 * twidth, 0, twidth * 12, font->height);
+		const auto rect = QRect(-5 * twidth, 0, twidth * 12, height);
+		if (badge.border.alpha() > 0) {
+			p.setPen(badge.border);
+		} else {
+			p.setPen(Qt::NoPen);
+		}
 		if (badge.gradient) {
 			const auto skip = font->height / M_SQRT2;
 			auto gradient = QLinearGradient(
@@ -784,4 +908,143 @@ QImage ValidateRotatedBadge(const GiftBadge &badge, int added) {
 	return result;
 }
 
+void SelectGiftToUnpin(
+		std::shared_ptr<ChatHelpers::Show> show,
+		const std::vector<Data::CreditsHistoryEntry> &pinned,
+		Fn<void(Data::SavedStarGiftId)> chosen) {
+	show->show(Box([=](not_null<Ui::GenericBox*> box) {
+		struct State {
+			explicit State(not_null<Main::Session*> session)
+			: delegate(session, GiftButtonMode::Minimal) {
+			}
+
+			Delegate delegate;
+			rpl::variable<int> selected = -1;
+			std::vector<not_null<GiftButton*>> buttons;
+		};
+		const auto session = &show->session();
+		const auto state = box->lifetime().make_state<State>(session);
+
+		box->setStyle(st::giftTooManyPinnedBox);
+		box->setWidth(st::boxWideWidth);
+
+		box->addRow(
+			object_ptr<Ui::FlatLabel>(
+				box,
+				tr::lng_gift_many_pinned_title(),
+				st::giftBoxSubtitle),
+			st::giftBoxSubtitleMargin);
+		box->addRow(
+			object_ptr<Ui::FlatLabel>(
+				box,
+				tr::lng_gift_many_pinned_choose(),
+				st::giftTooManyPinnedChoose),
+			st::giftBoxAboutMargin);
+
+		const auto gifts = box->addRow(
+			object_ptr<Ui::RpWidget>(box),
+			QMargins(
+				st::giftBoxPadding.left(),
+				st::giftTooManyPinnedBox.buttonPadding.top(),
+				st::giftBoxPadding.right(),
+				0));
+		for (const auto &entry : pinned) {
+			const auto index = int(state->buttons.size());
+			state->buttons.push_back(
+				Ui::CreateChild<GiftButton>(gifts, &state->delegate));
+			const auto button = state->buttons.back();
+			button->setDescriptor(GiftTypeStars{
+				.info = {
+					.id = entry.stargiftId,
+					.unique = entry.uniqueGift,
+					.document = entry.uniqueGift->model.document,
+				},
+				.pinnedSelection = true,
+			}, GiftButton::Mode::Minimal);
+			button->setClickedCallback([=] {
+				const auto now = state->selected.current();
+				state->selected = (now == index) ? -1 : index;
+			});
+		}
+
+		state->selected.value(
+		) | rpl::combine_previous(
+		) | rpl::start_with_next([=](int old, int now) {
+			if (old >= 0) state->buttons[old]->toggleSelected(false);
+			if (now >= 0) state->buttons[now]->toggleSelected(true);
+		}, gifts->lifetime());
+
+		gifts->widthValue() | rpl::start_with_next([=](int width) {
+			const auto singleMin = state->delegate.buttonSize();
+			if (width < singleMin.width()) {
+				return;
+			}
+			const auto count = int(state->buttons.size());
+			const auto skipw = st::giftBoxGiftSkip.x();
+			const auto skiph = st::giftBoxGiftSkip.y();
+			const auto perRow = std::min(
+				(width + skipw) / (singleMin.width() + skipw),
+				std::max(count, 1));
+			if (perRow <= 0) {
+				return;
+			}
+			const auto single = (width - (perRow - 1) * skipw) / perRow;
+			const auto height = singleMin.height();
+			const auto rows = (count + perRow - 1) / perRow;
+			for (auto row = 0; row != rows; ++row) {
+				const auto y = row * (height + skiph);
+				for (auto column = 0; column != perRow; ++column) {
+					const auto index = row * perRow + column;
+					if (index >= count) {
+						break;
+					}
+					const auto &button = state->buttons[index];
+					const auto x = column * (single + skipw);
+					button->setGeometry(
+						QRect(x, y, single, height),
+						state->delegate.buttonExtend());
+				}
+			}
+			gifts->resize(width, rows * (height + skiph) - skiph);
+		}, gifts->lifetime());
+
+		const auto button = box->addButton(rpl::single(QString()), [=] {
+			const auto index = state->selected.current();
+			if (index < 0) {
+				return;
+			}
+			Assert(index < int(pinned.size()));
+			const auto &entry = pinned[index];
+			const auto weak = Ui::MakeWeak(box);
+			chosen(::Settings::EntryToSavedStarGiftId(session, entry));
+			if (const auto strong = weak.data()) {
+				strong->closeBox();
+			}
+		});
+		const auto label = Ui::SetButtonMarkedLabel(
+			button,
+			tr::lng_context_unpin_from_top(Ui::Text::WithEntities),
+			&show->session(),
+			st::creditsBoxButtonLabel,
+			&st::giftTooManyPinnedBox.button.textFg);
+
+		state->selected.value() | rpl::start_with_next([=](int value) {
+			const auto has = (value >= 0);
+			label->setOpacity(has ? 1. : 0.5);
+			button->setAttribute(Qt::WA_TransparentForMouseEvents, !has);
+		}, box->lifetime());
+
+		const auto buttonPadding = st::giftTooManyPinnedBox.buttonPadding;
+		const auto buttonWidth = st::boxWideWidth
+			- buttonPadding.left()
+			- buttonPadding.right();
+		button->resizeToWidth(buttonWidth);
+		button->widthValue() | rpl::start_with_next([=](int width) {
+			if (width != buttonWidth) {
+				button->resizeToWidth(buttonWidth);
+			}
+		}, button->lifetime());
+	}));
+}
+
 } // namespace Info::PeerGifts
