@@ -45,6 +45,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_forum.h"
 #include "data/data_forum_topic.h"
 #include "data/data_message_reactions.h"
+#include "data/data_saved_sublist.h"
 #include "data/data_session.h"
 #include "main/main_session.h"
 #include "apiwrap.h"
@@ -924,13 +925,16 @@ void SetupUnreadReactionsMenu(
 			return;
 		}
 		const auto topic = thread->asTopic();
+		const auto sublist = thread->asSublist();
 		const auto peer = thread->peer();
 		const auto rootId = topic ? topic->rootId() : 0;
 		using Flag = MTPmessages_ReadReactions::Flag;
 		peer->session().api().request(MTPmessages_ReadReactions(
-			MTP_flags(rootId ? Flag::f_top_msg_id : Flag(0)),
+			MTP_flags((rootId ? Flag::f_top_msg_id : Flag(0))
+				| (sublist ? Flag::f_saved_peer_id : Flag(0))),
 			peer->input,
-			MTP_int(rootId)
+			MTP_int(rootId),
+			sublist ? sublist->sublistPeer()->input : MTPInputPeer()
 		)).done([=](const MTPmessages_AffectedHistory &result) {
 			const auto offset = peer->session().api().applyAffectedHistory(
 				peer,
@@ -939,7 +943,9 @@ void SetupUnreadReactionsMenu(
 				resend(weakThread, done, resend);
 			} else {
 				done();
-				peer->owner().history(peer)->clearUnreadReactionsFor(rootId);
+				peer->owner().history(peer)->clearUnreadReactionsFor(
+					rootId,
+					sublist);
 			}
 		}).fail(done).send();
 	};
