@@ -58,6 +58,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/painter.h"
 #include "window/themes/window_theme.h"
 #include "window/section_widget.h"
+#include "window/window_peer_menu.h"
 #include "window/window_session_controller.h"
 #include "styles/style_boxes.h"
 #include "styles/style_chat_helpers.h"
@@ -373,7 +374,7 @@ ShortcutMessages::ShortcutMessages(
 		this,
 		&controller->session(),
 		static_cast<ListDelegate*>(this));
-	_inner->overrideIsChatWide(false);
+	_inner->overrideChatMode(ElementChatMode::Default);
 
 	_scroll->sizeValue() | rpl::filter([](QSize size) {
 		return !size.isEmpty();
@@ -399,6 +400,8 @@ ShortcutMessages::ShortcutMessages(
 				_composeControls->editMessage(
 					fullId,
 					_inner->getSelectedTextRange(item));
+			} else if (media->todolist()) {
+				Window::PeerMenuEditTodoList(_controller, item);
 			}
 		}
 	}, _inner->lifetime());
@@ -629,6 +632,7 @@ void ShortcutMessages::setupComposeControls() {
 		.key = Dialogs::Key{ _history },
 		.section = Dialogs::EntryState::Section::ShortcutMessages,
 		.currentReplyTo = replyTo(),
+		.currentSuggest = SuggestPostOptions(),
 	};
 	_composeControls->setCurrentDialogsEntryState(state);
 
