@@ -43,6 +43,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "info/bot/starref/info_bot_starref_common.h"
 #include "info/channel_statistics/boosts/giveaway/boost_badge.h" // InfiniteRadialAnimationWidget.
 #include "info/channel_statistics/earn/info_channel_earn_widget.h" // Info::ChannelEarn::Make.
+#include "info/channel_statistics/earn/earn_format.h"
+#include "info/channel_statistics/earn/earn_icons.h"
+#include "info/peer_gifts/info_peer_gifts_common.h"
 #include "info/settings/info_settings_widget.h" // SectionCustomTopBarData.
 #include "info/statistics/info_statistics_list_controllers.h"
 #include "info/info_controller.h"
@@ -140,13 +143,13 @@ class Balance final
 public:
 	using Ui::RpWidget::RpWidget;
 
-	void setBalance(StarsAmount balance) {
+	void setBalance(CreditsAmount balance) {
 		_balance = balance;
-		_tooltip = Lang::FormatStarsAmountDecimal(balance);
+		_tooltip = Lang::FormatCreditsAmountDecimal(balance);
 	}
 
 	void enterEventHook(QEnterEvent *e) override {
-		if (_balance >= StarsAmount(10'000)) {
+		if (_balance >= CreditsAmount(10'000)) {
 			Ui::Tooltip::Show(1000, this);
 		}
 	}
@@ -169,22 +172,10 @@ public:
 
 private:
 	QString _tooltip;
-	StarsAmount _balance;
+	CreditsAmount _balance;
 
 };
 
-[[nodiscard]] Data::SavedStarGiftId EntryToSavedStarGiftId(
-		not_null<Main::Session*> session,
-		const Data::CreditsHistoryEntry &entry) {
-	return !entry.stargift
-		? Data::SavedStarGiftId()
-		: (entry.bareEntryOwnerId && entry.giftChannelSavedId)
-		? Data::SavedStarGiftId::Chat(
-			session->data().peer(PeerId(entry.bareEntryOwnerId)),
-			entry.giftChannelSavedId)
-		: Data::SavedStarGiftId::User(MsgId(entry.bareMsgId));
-}
-
 void ToggleStarGiftSaved(
 		std::shared_ptr<ChatHelpers::Show> show,
 		Data::SavedStarGiftId savedId,
@@ -226,7 +217,8 @@ void ToggleStarGiftPinned(
 		Data::SavedStarGiftId savedId,
 		std::vector<Data::SavedStarGiftId> already,
 		bool pinned,
-		Fn<void(bool)> done = nullptr) {
+		std::shared_ptr<Data::UniqueGift> uniqueData = nullptr,
+		std::shared_ptr<Data::UniqueGift> replacingData = nullptr) {
 	already.erase(ranges::remove(already, savedId), end(already));
 	if (pinned) {
 		already.insert(begin(already), savedId);
@@ -256,16 +248,29 @@ void ToggleStarGiftPinned(
 			.action = (pinned ? GiftAction::Pin : GiftAction::Unpin),
 		});
 
-		if (const auto onstack = done) {
-			onstack(true);
-		}
 		if (pinned) {
-			show->showToast(tr::lng_gift_pinned_done(tr::now));
+			show->showToast({
+				.title = (uniqueData
+					? tr::lng_gift_pinned_done_title(
+						tr::now,
+						lt_gift,
+						Data::UniqueGiftName(*uniqueData))
+					: QString()),
+				.text = (replacingData
+					? tr::lng_gift_pinned_done_replaced(
+						tr::now,
+						lt_gift,
+						TextWithEntities{
+							Data::UniqueGiftName(*replacingData),
+						},
+						Ui::Text::WithEntities)
+					: tr::lng_gift_pinned_done(
+						tr::now,
+						Ui::Text::WithEntities)),
+				.duration = Ui::Toast::kDefaultDuration * 2,
+			});
 		}
 	}).fail([=](const MTP::Error &error) {
-		if (const auto onstack = done) {
-			onstack(false);
-		}
 		show->showToast(error.type());
 	}).send();
 }
@@ -438,7 +443,7 @@ void AddMiniStars(
 	const auto stars = widget->lifetime().make_state<ColoredMiniStars>(
 		widget,
 		false,
-		Ui::Premium::MiniStars::Type::BiStars);
+		Ui::Premium::MiniStarsType::BiStars);
 	stars->setColorOverride(Ui::Premium::CreditsIconGradientStops());
 	widget->resize(boxWidth - photoSize, photoSize * heightRatio);
 	content->sizeValue(
@@ -503,7 +508,7 @@ void FillCreditOptions(
 		std::shared_ptr<Main::SessionShow> show,
 		not_null<Ui::VerticalLayout*> container,
 		not_null<PeerData*> peer,
-		StarsAmount minimumCredits,
+		CreditsAmount minimumCredits,
 		Fn<void()> paid,
 		rpl::producer<QString> subtitle,
 		std::vector<Data::CreditTopupOption> preloadedTopupOptions) {
@@ -549,12 +554,12 @@ void FillCreditOptions(
 			- int(singleStarWidth * 1.5);
 		const auto buttonHeight = st.height + rect::m::sum::v(st.padding);
 		const auto minCredits = (!options.empty()
-				&& (minimumCredits > StarsAmount(options.back().credits)))
-			? StarsAmount()
+				&& (minimumCredits > CreditsAmount(options.back().credits)))
+			? CreditsAmount()
 			: minimumCredits;
 		for (auto i = 0; i < options.size(); i++) {
 			const auto &option = options[i];
-			if (StarsAmount(option.credits) < minCredits) {
+			if (CreditsAmount(option.credits) < minCredits) {
 				continue;
 			}
 			const auto button = [&] {
@@ -681,19 +686,17 @@ void FillCreditOptions(
 
 not_null<Ui::RpWidget*> AddBalanceWidget(
 		not_null<Ui::RpWidget*> parent,
-		rpl::producer<StarsAmount> balanceValue,
+		not_null<Main::Session*> session,
+		rpl::producer<CreditsAmount> balanceValue,
 		bool rightAlign,
 		rpl::producer<float64> opacityValue) {
 	struct State final {
-		QImage star;
 		float64 opacity = 1.0;
 		Ui::Text::String label;
 		Ui::Text::String count;
 	};
 	const auto balance = Ui::CreateChild<Balance>(parent);
 	const auto state = balance->lifetime().make_state<State>();
-	state->star = QImage(Ui::GenerateStars(st::creditsBalanceStarHeight, 1));
-	const auto starSize = state->star.size() / style::DevicePixelRatio();
 	state->label = Ui::Text::String(
 		st::defaultTextStyle,
 		tr::lng_credits_summary_balance(tr::now));
@@ -705,22 +708,40 @@ not_null<Ui::RpWidget*> AddBalanceWidget(
 			state->opacity = value;
 		}, balance->lifetime());
 	}
-	const auto diffBetweenStarAndCount = state->count.style()->font->spacew;
 	const auto resize = [=] {
 		balance->resize(
-			std::max(
-				state->label.maxWidth(),
-				state->count.maxWidth()
-					+ starSize.width()
-					+ diffBetweenStarAndCount),
-			state->label.style()->font->height + starSize.height());
+			std::max(state->label.maxWidth(), state->count.maxWidth()),
+			(state->label.style()->font->height
+				+ state->count.style()->font->height));
 	};
 	std::move(
 		balanceValue
-	) | rpl::start_with_next([=](StarsAmount value) {
-		state->count.setText(
+	) | rpl::start_with_next([=](CreditsAmount value) {
+		auto text = TextWithEntities();
+		const auto manager = &session->data().customEmojiManager();
+		if (value.ton()) {
+			text.append(Ui::Text::SingleCustomEmoji(
+				manager->registerInternalEmoji(
+					Ui::Earn::IconCurrencyColored(
+						st::tonFieldIconSize,
+						st::currencyFg->c),
+					st::channelEarnCurrencyLearnMargins,
+					false))
+			).append(' ').append(Lang::FormatCreditsAmountDecimal(value));
+		} else {
+			text.append(
+				manager->creditsEmoji()
+			).append(' ').append(
+				Lang::FormatCreditsAmountToShort(value).string);
+		}
+		state->count.setMarkedText(
 			st::semiboldTextStyle,
-			Lang::FormatStarsAmountToShort(value).string);
+			text,
+			kMarkupTextOptions,
+			Core::TextContext({
+				.session = session,
+				.repaint = [=] { balance->update(); },
+			}));
 		balance->setBalance(value);
 		resize();
 	}, balance->lifetime());
@@ -739,22 +760,10 @@ not_null<Ui::RpWidget*> AddBalanceWidget(
 		});
 		state->count.draw(p, {
 			.position = QPoint(
-				(rightAlign
-					? (balance->width() - state->count.maxWidth())
-					: (starSize.width() + diffBetweenStarAndCount)),
-				state->label.minHeight()
-					+ (starSize.height() - state->count.minHeight()) / 2),
+				rightAlign ? (balance->width() - state->count.maxWidth()) : 0,
+				state->label.minHeight()),
 			.availableWidth = balance->width(),
 		});
-		p.drawImage(
-			(rightAlign
-				? (balance->width()
-					- state->count.maxWidth()
-					- starSize.width()
-					- diffBetweenStarAndCount)
-				: 0),
-			state->label.minHeight(),
-			state->star);
 	}, balance->lifetime());
 	return balance;
 }
@@ -901,6 +910,22 @@ void ProcessReceivedSubscriptions(
 	}
 }
 
+[[nodiscard]] bool CanResellGift(
+		not_null<Main::Session*> session,
+		const Data::CreditsHistoryEntry &e) {
+	const auto unique = e.uniqueGift.get();
+	const auto owner = unique
+		? session->data().peer(unique->ownerId).get()
+		: nullptr;
+	return !owner
+		? false
+		: owner->isSelf()
+		? e.in
+		: false;
+	// Currently we're not reselling channel gifts.
+	// (owner->isChannel() && owner->asChannel()->canTransferGifts());
+}
+
 void FillUniqueGiftMenu(
 		std::shared_ptr<ChatHelpers::Show> show,
 		not_null<Ui::PopupMenu*> menu,
@@ -920,25 +945,61 @@ void FillUniqueGiftMenu(
 	if (unique
 		&& canToggle
 		&& e.savedToProfile
-		&& type == SavedStarGiftMenuType::List) {
-		const auto already = [session, entries = e.pinnedSavedGifts] {
-			Expects(entries != nullptr);
-
-			auto list = entries();
+		&& e.pinnedSavedGifts) {
+		const auto pinned = e.pinnedSavedGifts;
+		const auto ids = [session](
+				const std::vector<Data::CreditsHistoryEntry> &pinned) {
 			auto result = std::vector<Data::SavedStarGiftId>();
-			result.reserve(list.size());
-			for (const auto &entry : list) {
+			result.reserve(pinned.size());
+			for (const auto &entry : pinned) {
 				result.push_back(EntryToSavedStarGiftId(session, entry));
 			}
 			return result;
 		};
 		if (e.giftPinned) {
 			menu->addAction(tr::lng_context_unpin_from_top(tr::now), [=] {
-				ToggleStarGiftPinned(show, savedId, already(), false);
+				ToggleStarGiftPinned(show, savedId, ids(pinned()), false);
 			}, st.unpin ? st.unpin : &st::menuIconUnpin);
 		} else {
 			menu->addAction(tr::lng_context_pin_to_top(tr::now), [=] {
-				ToggleStarGiftPinned(show, savedId, already(), true);
+				const auto list = pinned();
+				const auto &appConfig = show->session().appConfig();
+				const auto limit = appConfig.pinnedGiftsLimit();
+				auto already = ids(list);
+				if (list.size() >= limit) {
+					Info::PeerGifts::SelectGiftToUnpin(show, list, [=](
+							Data::SavedStarGiftId id) {
+						auto copy = already;
+						const auto i = ranges::find(copy, id);
+						const auto replaced = (i != end(copy))
+							? list[i - begin(copy)].uniqueGift
+							: nullptr;
+						if (i != end(copy)) {
+							copy.erase(i);
+						}
+
+						using GiftAction = Data::GiftUpdate::Action;
+						show->session().data().notifyGiftUpdate({
+							.id = id,
+							.action = GiftAction::Unpin,
+						});
+
+						ToggleStarGiftPinned(
+							show,
+							savedId,
+							already,
+							true,
+							unique,
+							replaced);
+					});
+				} else {
+					ToggleStarGiftPinned(
+						show,
+						savedId,
+						already,
+						true,
+						unique);
+				}
 			}, st.pin ? st.pin : &st::menuIconPin);
 		}
 	}
@@ -1003,11 +1064,41 @@ void FillUniqueGiftMenu(
 			}, st.wear ? st.wear : &st::menuIconNftWear);
 		}
 	}
+	if (CanResellGift(&show->session(), e)) {
+		const auto resalePrice = unique->starsForResale;
+		const auto editPrice = (resalePrice > 0
+			? tr::lng_gift_transfer_update
+			: tr::lng_gift_transfer_sell)(tr::now);
+		menu->addAction(editPrice, [=] {
+			const auto style = st.giftWearBox
+				? *st.giftWearBox
+				: GiftWearBoxStyleOverride();
+			ShowUniqueGiftSellBox(show, unique, savedId, style);
+		}, st.resell ? st.resell : &st::menuIconTagSell);
+		if (resalePrice > 0) {
+			menu->addAction(tr::lng_gift_transfer_unlist(tr::now), [=] {
+				const auto name = UniqueGiftName(*unique);
+				const auto confirm = [=](Fn<void()> close) {
+					close();
+					Ui::UpdateGiftSellPrice(show, unique, savedId, 0);
+				};
+				show->show(Ui::MakeConfirmBox({
+					.text = tr::lng_gift_sell_unlist_sure(),
+					.confirmed = confirm,
+					.confirmText = tr::lng_gift_transfer_unlist(),
+					.title = tr::lng_gift_sell_unlist_title(
+						lt_name,
+						rpl::single(name)),
+				}));
+			}, st.unlist ? st.unlist : &st::menuIconTagRemove);
+		}
+	}
 }
 
 GiftWearBoxStyleOverride DarkGiftWearBoxStyle() {
 	return {
 		.box = &st::darkUpgradeGiftBox,
+		.close = &st::darkGiftBoxClose,
 		.title = &st::darkUpgradeGiftTitle,
 		.subtitle = &st::darkUpgradeGiftSubtitle,
 		.radiantIcon = &st::darkUpgradeGiftRadiant,
@@ -1029,6 +1120,8 @@ CreditsEntryBoxStyleOverrides DarkCreditsEntryBoxStyle() {
 		.transfer = &st::darkGiftTransfer,
 		.wear = &st::darkGiftNftWear,
 		.takeoff = &st::darkGiftNftTakeOff,
+		.resell = &st::darkGiftNftResell,
+		.unlist = &st::darkGiftNftUnlist,
 		.show = &st::darkGiftShow,
 		.hide = &st::darkGiftHide,
 		.pin = &st::darkGiftPin,
@@ -1070,7 +1163,8 @@ void GenericCreditsEntryBox(
 		&& giftChannel->canTransferGifts();
 	const auto starGiftCanManage = isStarGift
 		&& !creditsHistoryStarGift
-		&& (e.in || giftToChannelCanManage);
+		&& (e.in || giftToChannelCanManage)
+		&& !e.fromGiftSlug;
 	const auto starGiftCanTransfer = isStarGift
 		&& !creditsHistoryStarGift
 		&& (e.in || giftToChannelCanTransfer);
@@ -1091,11 +1185,14 @@ void GenericCreditsEntryBox(
 		&& !e.converted
 		&& starGiftSender;
 	const auto canConvert = forConvert && !timeExceeded;
+	const auto inResale = uniqueGift && (uniqueGift->starsForResale > 0);
+	const auto canBuyResold = inResale && (e.bareGiftOwnerId != selfPeerId);
 
 	if (auto savedId = EntryToSavedStarGiftId(session, e)) {
 		session->data().giftUpdates(
 		) | rpl::start_with_next([=](const Data::GiftUpdate &update) {
-			if (update.id == savedId) {
+			if (update.id == savedId
+				&& update.action != Data::GiftUpdate::Action::ResaleChange) {
 				box->closeBox();
 			}
 		}, box->lifetime());
@@ -1133,7 +1230,32 @@ void GenericCreditsEntryBox(
 	if (uniqueGift) {
 		box->setNoContentMargin(true);
 
-		AddUniqueGiftCover(content, rpl::single(*uniqueGift));
+		const auto slug = uniqueGift->slug;
+		auto price = rpl::single(
+			rpl::empty
+		) | rpl::then(session->data().giftUpdates(
+		) | rpl::filter([=](const Data::GiftUpdate &update) {
+			return (update.action == Data::GiftUpdate::Action::ResaleChange)
+				&& (update.slug == slug);
+		}) | rpl::to_empty) | rpl::map([unique = e.uniqueGift] {
+			return unique->starsForResale;
+		});
+		auto change = [=] {
+			const auto style = st.giftWearBox
+				? *st.giftWearBox
+				: GiftWearBoxStyleOverride();
+			ShowUniqueGiftSellBox(
+				show,
+				e.uniqueGift,
+				EntryToSavedStarGiftId(session, e),
+				style);
+		};
+		AddUniqueGiftCover(
+			content,
+			rpl::single(*uniqueGift),
+			{},
+			std::move(price),
+			CanResellGift(session, e) ? std::move(change) : Fn<void()>());
 
 		AddSkip(content, st::defaultVerticalListSkip * 2);
 
@@ -1189,14 +1311,22 @@ void GenericCreditsEntryBox(
 		auto &packs = session->giftBoxStickersPacks();
 		const auto document = starGiftSticker
 			? starGiftSticker
-			: packs.lookup(e.premiumMonthsForStars
-				? e.premiumMonthsForStars
-				: packs.monthsForStars(e.credits.whole()));
+			: e.credits.ton()
+			? packs.tonLookup(e.credits.whole())
+			: packs.lookup(
+				e.premiumMonthsForStars
+					? e.premiumMonthsForStars
+					: packs.monthsForStars(e.credits.whole()));
 		if (document && document->sticker()) {
+			const auto origin = starGiftSticker
+				? starGiftSticker->stickerOrGifOrigin()
+				: e.credits.ton()
+				? packs.tonOrigin()
+				: packs.origin();
 			state->sticker = document;
 			state->media = document->createMediaView();
-			state->media->thumbnailWanted(packs.origin());
-			state->media->automaticLoad(packs.origin(), nullptr);
+			state->media->thumbnailWanted(origin);
+			state->media->automaticLoad(origin, nullptr);
 			rpl::single() | rpl::then(
 				session->downloaderTaskFinished()
 			) | rpl::filter([=] {
@@ -1308,6 +1438,7 @@ void GenericCreditsEntryBox(
 		constexpr auto kMinus = QChar(0x2212);
 		auto &lifetime = content->lifetime();
 		const auto text = lifetime.make_state<Ui::Text::String>();
+		auto minorText = (Ui::Text::String*)(nullptr);
 		const auto roundedText = e.refunded
 			? tr::lng_channel_earn_history_return(tr::now)
 			: e.pending
@@ -1345,14 +1476,14 @@ void GenericCreditsEntryBox(
 					Ui::Text::WithEntities),
 				kMarkupTextOptions,
 				context);
-		} else {
+		} else if (e.credits.stars()) {
 			auto t = TextWithEntities()
 				.append((e.in && (creditsHistoryStarGift || !isStarGift))
-					? u"+"_q
+					? QChar('+')
 					: (e.gift && !creditsHistoryStarGift)
-					? QString()
-					: QString(kMinus))
-				.append(Lang::FormatStarsAmountDecimal(e.credits.abs()))
+					? QChar()
+					: kMinus)
+				.append(Lang::FormatCreditsAmountDecimal(e.credits.abs()))
 				.append(QChar(' '))
 				.append(owner->customEmojiManager().creditsEmoji());
 			text->setMarkedText(
@@ -1360,6 +1491,35 @@ void GenericCreditsEntryBox(
 				std::move(t),
 				kMarkupTextOptions,
 				context);
+		} else if (e.credits.ton()) {
+			auto t = TextWithEntities()
+				.append((e.in ? QChar('+') : kMinus))
+				.append(Info::ChannelEarn::MajorPart(e.credits.abs()));
+			text->setMarkedText(
+				st::channelEarnHistoryMajorLabel.style,
+				std::move(t),
+				kMarkupTextOptions,
+				context);
+
+			auto minor = TextWithEntities()
+				.append(Info::ChannelEarn::MinorPart(e.credits.abs()))
+				.append(QChar(' '))
+				.append(
+					Ui::Text::SingleCustomEmoji(
+						owner->customEmojiManager().registerInternalEmoji(
+							Ui::Earn::IconCurrencyColored(
+								st::tonFieldIconSize,
+								(e.in
+									? st::boxTextFgGood->c
+									: st::menuIconAttentionColor->c)),
+							QMargins(0, st::lineWidth * 1, 0, 0),
+							false)));
+			minorText = lifetime.make_state<Ui::Text::String>();
+			minorText->setMarkedText(
+				st::channelEarnHistoryMinorLabel.style,
+				std::move(minor),
+				kMarkupTextOptions,
+				context);
 		}
 		const auto font = text->style()->font;
 		const auto roundedFont = st::defaultTextStyle.font;
@@ -1369,7 +1529,9 @@ void GenericCreditsEntryBox(
 				+ roundedSkip
 				+ roundedFont->height
 			: 0;
-		const auto fullWidth = text->maxWidth() + roundedWidth;
+		const auto fullWidth = text->maxWidth()
+			+ roundedWidth
+			+ (minorText ? minorText->maxWidth() : 0);
 		amount->paintRequest(
 		) | rpl::start_with_next([=] {
 			auto p = Painter(amount);
@@ -1385,13 +1547,21 @@ void GenericCreditsEntryBox(
 				? st::windowBoldFg
 				: st::menuIconAttentionColor);
 			const auto x = (amount->width() - fullWidth) / 2;
+			const auto y = (amount->height() - font->height) / 2;
 			text->draw(p, Ui::Text::PaintContext{
-				.position = QPoint(
-					x,
-					(amount->height() - font->height) / 2),
+				.position = QPoint(x, y),
 				.outerWidth = amount->width(),
 				.availableWidth = amount->width(),
 			});
+			if (minorText) {
+				minorText->draw(p, Ui::Text::PaintContext{
+					.position = QPoint(
+						x + text->maxWidth(),
+						y + st::lineWidth * 2),
+					.outerWidth = amount->width(),
+					.availableWidth = amount->width(),
+				});
+			}
 
 			if (rounded) {
 				const auto roundedLeft = fullWidth
@@ -1499,7 +1669,7 @@ void GenericCreditsEntryBox(
 			about->setTextColorOverride(st::menuIconAttentionColor->c);
 		}
 	} else if (isStarGift) {
-	} else if (e.gift || isPrize) {
+	} else if ((e.gift || isPrize) && e.credits.stars()) {
 		Ui::AddSkip(content);
 		auto link = tr::lng_credits_box_history_entry_gift_about_link(
 			lt_emoji,
@@ -1684,7 +1854,7 @@ void GenericCreditsEntryBox(
 
 	Ui::AddSkip(content);
 
-	if (!isStarGift) {
+	if (!isStarGift && e.credits.stars()) {
 		box->addRow(object_ptr<Ui::CenterWrap<>>(
 			box,
 			object_ptr<Ui::FlatLabel>(
@@ -1832,7 +2002,7 @@ void GenericCreditsEntryBox(
 				: tr::lng_gift_show_on_page)()
 			: tr::lng_box_ok()));
 	const auto send = [=, weak = Ui::MakeWeak(box)] {
-		if (toRejoin) {
+		if (toRejoin && !toRenew) {
 			if (const auto window = show->resolveWindow()) {
 				const auto finish = [=](Payments::CheckoutResult&&) {
 					ProcessReceivedSubscriptions(weak, session);
@@ -1856,7 +2026,7 @@ void GenericCreditsEntryBox(
 				ProcessReceivedSubscriptions(weak, session);
 			};
 			const auto fail = [=, show = box->uiShow()](const QString &e) {
-				if (const auto strong = weak.data()) {
+				if ([[maybe_unused]] const auto strong = weak.data()) {
 					state->confirmButtonBusy = false;
 				}
 				show->showToast(e);
@@ -1884,6 +2054,16 @@ void GenericCreditsEntryBox(
 		if (willBusy) {
 			state->confirmButtonBusy = true;
 			send();
+		} else if (canBuyResold) {
+			const auto to = e.bareGiftResaleRecipientId
+				? show->session().data().peer(
+					PeerId(e.bareGiftResaleRecipientId))
+				: show->session().user();
+			ShowBuyResaleGiftBox(
+				show,
+				e.uniqueGift,
+				to,
+				crl::guard(box, [=] { box->closeBox(); }));
 		} else if (canUpgradeFree) {
 			upgrade();
 		} else if (canToggle && !e.savedToProfile) {
@@ -1892,6 +2072,14 @@ void GenericCreditsEntryBox(
 			box->closeBox();
 		}
 	});
+	if (canBuyResold) {
+		button->setText(tr::lng_gift_buy_resale_button(
+			lt_cost,
+			rpl::single(
+				Ui::Text::IconEmoji(&st::starIconEmoji).append(
+					Lang::FormatCountDecimal(uniqueGift->starsForResale))),
+			Ui::Text::WithEntities));
+	}
 	{
 		using namespace Info::Statistics;
 		const auto loadingAnimation = InfiniteRadialAnimationWidget(
@@ -1935,7 +2123,7 @@ void GiftedCreditsBox(
 			? tr::lng_credits_box_history_entry_gift_name
 			: tr::lng_credits_box_history_entry_gift_sent)(tr::now),
 		.date = base::unixtime::parse(date),
-		.credits = StarsAmount(count),
+		.credits = CreditsAmount(count),
 		.bareMsgId = uint64(),
 		.barePeerId = (anonymous ? uint64() : peer->id.value),
 		.peerType = (anonymous ? PeerType::Fragment : PeerType::Peer),
@@ -1958,7 +2146,7 @@ void CreditsPrizeBox(
 			.title = QString(),
 			.description = TextWithEntities(),
 			.date = base::unixtime::parse(date),
-			.credits = StarsAmount(data.count),
+			.credits = CreditsAmount(data.count),
 			.barePeerId = data.channel
 				? data.channel->id.value
 				: 0,
@@ -1973,15 +2161,20 @@ void GlobalStarGiftBox(
 		not_null<Ui::GenericBox*> box,
 		std::shared_ptr<ChatHelpers::Show> show,
 		const Data::StarGift &data,
+		PeerId resaleRecipientId,
 		CreditsEntryBoxStyleOverrides st) {
+	const auto selfId = show->session().userPeerId();
 	const auto ownerId = data.unique ? data.unique->ownerId.value : 0;
 	Settings::GenericCreditsEntryBox(
 		box,
 		show,
 		Data::CreditsHistoryEntry{
-			.credits = StarsAmount(data.stars),
+			.credits = CreditsAmount(data.stars),
 			.bareGiftStickerId = data.document->id,
 			.bareGiftOwnerId = ownerId,
+			.bareGiftResaleRecipientId = ((resaleRecipientId != selfId)
+				? resaleRecipientId.value
+				: 0),
 			.stargiftId = data.id,
 			.uniqueGift = data.unique,
 			.peerType = Data::CreditsHistoryEntry::PeerType::Peer,
@@ -2003,7 +2196,7 @@ Data::CreditsHistoryEntry SavedStarGiftEntry(
 	return {
 		.description = data.message,
 		.date = base::unixtime::parse(data.date),
-		.credits = StarsAmount(data.info.stars),
+		.credits = CreditsAmount(data.info.stars),
 		.bareMsgId = uint64(data.manageId.userMessageId().bare),
 		.barePeerId = data.fromId.value,
 		.bareGiftStickerId = data.info.document->id,
@@ -2031,15 +2224,30 @@ Data::CreditsHistoryEntry SavedStarGiftEntry(
 	};
 }
 
+Data::SavedStarGiftId EntryToSavedStarGiftId(
+		not_null<Main::Session*> session,
+		const Data::CreditsHistoryEntry &entry) {
+	return !entry.stargift
+		? Data::SavedStarGiftId()
+		: (entry.bareEntryOwnerId && entry.giftChannelSavedId)
+		? Data::SavedStarGiftId::Chat(
+			session->data().peer(PeerId(entry.bareEntryOwnerId)),
+			entry.giftChannelSavedId)
+		: Data::SavedStarGiftId::User(MsgId(entry.bareMsgId));
+}
+
 void SavedStarGiftBox(
 		not_null<Ui::GenericBox*> box,
 		not_null<Window::SessionController*> controller,
 		not_null<PeerData*> owner,
-		const Data::SavedStarGift &data) {
+		const Data::SavedStarGift &data,
+		Fn<std::vector<Data::CreditsHistoryEntry>()> pinned) {
+	auto entry = SavedStarGiftEntry(owner, data);
+	entry.pinnedSavedGifts = std::move(pinned);
 	Settings::ReceiptCreditsBox(
 		box,
 		controller,
-		SavedStarGiftEntry(owner, data),
+		std::move(entry),
 		Data::SubscriptionEntry());
 }
 
@@ -2067,7 +2275,7 @@ void StarGiftViewBox(
 		.id = data.slug,
 		.description = data.message,
 		.date = base::unixtime::parse(item->date()),
-		.credits = StarsAmount(data.count),
+		.credits = CreditsAmount(data.count),
 		.bareMsgId = uint64(item->id.bare),
 		.barePeerId = fromId.value,
 		.bareGiftStickerId = data.document ? data.document->id : 0,
@@ -2118,7 +2326,7 @@ void ShowRefundInfoBox(
 	auto info = Data::CreditsHistoryEntry();
 	info.id = refund->transactionId;
 	info.date = base::unixtime::parse(item->date());
-	info.credits = StarsAmount(refund->amount);
+	info.credits = CreditsAmount(refund->amount);
 	info.barePeerId = refund->peer->id.value;
 	info.peerType = Data::CreditsHistoryEntry::PeerType::Peer;
 	info.refunded = true;
@@ -2216,7 +2424,7 @@ void SmallBalanceBox(
 		Fn<void()> paid) {
 	Expects(show->session().credits().loaded());
 
-	auto credits = StarsAmount(wholeCredits);
+	auto credits = CreditsAmount(wholeCredits);
 
 	box->setWidth(st::boxWideWidth);
 	box->addButton(tr::lng_close(), [=] { box->closeBox(); });
@@ -2242,11 +2450,15 @@ void SmallBalanceBox(
 		return value.recipientId
 			? owner->peer(value.recipientId)->shortName()
 			: QString();
+	}, [&](SmallBalanceForSuggest value) {
+		return value.recipientId
+			? owner->peer(value.recipientId)->shortName()
+			: QString();
 	});
 
 	auto needed = show->session().credits().balanceValue(
-	) | rpl::map([=](StarsAmount balance) {
-		return (balance < credits) ? (credits - balance) : StarsAmount();
+	) | rpl::map([=](CreditsAmount balance) {
+		return (balance < credits) ? (credits - balance) : CreditsAmount();
 	});
 	const auto content = [&]() -> Ui::Premium::TopBarAbstract* {
 		return box->setPinnedToTopContent(object_ptr<Ui::Premium::TopBar>(
@@ -2258,8 +2470,8 @@ void SmallBalanceBox(
 					rpl::duplicate(
 						needed
 					) | rpl::filter(
-						rpl::mappers::_1 > StarsAmount(0)
-					) | rpl::map([](StarsAmount amount) {
+						rpl::mappers::_1 > CreditsAmount(0)
+					) | rpl::map([](CreditsAmount amount) {
 						return amount.value();
 					})),
 				.about = (v::is<SmallBalanceSubscription>(source)
@@ -2288,6 +2500,11 @@ void SmallBalanceBox(
 							lt_user,
 							rpl::single(Ui::Text::Bold(name)),
 							Ui::Text::RichLangValue))
+					: v::is<SmallBalanceForSuggest>(source)
+					? tr::lng_credits_small_balance_for_suggest(
+						lt_channel,
+						rpl::single(Ui::Text::Bold(name)),
+						Ui::Text::RichLangValue)
 					: name.isEmpty()
 					? tr::lng_credits_small_balance_fallback(
 						Ui::Text::RichLangValue)
@@ -2326,6 +2543,7 @@ void SmallBalanceBox(
 	{
 		const auto balance = AddBalanceWidget(
 			content,
+			&show->session(),
 			show->session().credits().balanceValue(),
 			true);
 		show->session().credits().load(true);
@@ -2353,7 +2571,7 @@ void AddWithdrawalWidget(
 		not_null<Window::SessionController*> controller,
 		not_null<PeerData*> peer,
 		rpl::producer<QString> secondButtonUrl,
-		rpl::producer<StarsAmount> availableBalanceValue,
+		rpl::producer<CreditsAmount> availableBalanceValue,
 		rpl::producer<QDateTime> dateValue,
 		bool withdrawalEnabled,
 		rpl::producer<QString> usdValue) {
@@ -2368,8 +2586,8 @@ void AddWithdrawalWidget(
 		labels,
 		rpl::duplicate(
 			availableBalanceValue
-		) | rpl::map([](StarsAmount v) {
-			return Lang::FormatStarsAmountDecimal(v);
+		) | rpl::map([](CreditsAmount v) {
+			return Lang::FormatCreditsAmountDecimal(v);
 		}),
 		st::channelEarnBalanceMajorLabel);
 	const auto icon = Ui::CreateSingleStarWidget(
@@ -2401,9 +2619,15 @@ void AddWithdrawalWidget(
 		object_ptr<Ui::SlideWrap<Ui::VerticalLayout>>(
 			container,
 			object_ptr<Ui::VerticalLayout>(container)));
+	const auto starsWithdrawMax = CreditsAmount(
+		controller->session().appConfig().starsWithdrawMax());
 	const auto input = Ui::AddInputFieldForCredits(
 		withdrawalWrap->entity(),
-		rpl::duplicate(availableBalanceValue));
+		rpl::duplicate(
+			availableBalanceValue
+		) | rpl::map([=](CreditsAmount amount) {
+			return (amount > starsWithdrawMax) ? starsWithdrawMax : amount;
+		}));
 
 	Ui::AddSkip(withdrawalWrap->entity());
 	Ui::AddSkip(withdrawalWrap->entity());
@@ -2468,7 +2692,7 @@ void AddWithdrawalWidget(
 				st::settingsPremiumIconStar,
 				{ 0, -st::moderateBoxExpandInnerSkip, 0, 0 },
 				true));
-		using Balance = rpl::variable<StarsAmount>;
+		using Balance = rpl::variable<CreditsAmount>;
 		const auto currentBalance = input->lifetime().make_state<Balance>(
 			rpl::duplicate(availableBalanceValue));
 		const auto process = [=] {
@@ -2636,17 +2860,19 @@ void AddWithdrawalWidget(
 	const auto arrow = Ui::Text::IconEmoji(&st::textMoreIconEmoji);
 	auto about = Ui::CreateLabelWithCustomEmoji(
 		container,
-		tr::lng_bot_earn_learn_credits_out_about(
-			lt_link,
-			tr::lng_channel_earn_about_link(
-				lt_emoji,
-				rpl::single(arrow),
-				Ui::Text::RichLangValue
-			) | rpl::map([](TextWithEntities text) {
-				return Ui::Text::Link(
-					std::move(text),
-					tr::lng_bot_earn_balance_about_url(tr::now));
-			}),
+		(peer->isSelf()
+			? tr::lng_self_earn_learn_credits_out_about
+			: tr::lng_bot_earn_learn_credits_out_about)(
+				lt_link,
+				tr::lng_channel_earn_about_link(
+					lt_emoji,
+					rpl::single(arrow),
+					Ui::Text::RichLangValue
+				) | rpl::map([](TextWithEntities text) {
+					return Ui::Text::Link(
+						std::move(text),
+						tr::lng_bot_earn_balance_about_url(tr::now));
+				}),
 			Ui::Text::RichLangValue),
 		Core::TextContext({ .session = session }),
 		st::boxDividerLabel);
@@ -2678,7 +2904,7 @@ void MaybeRequestBalanceIncrease(
 		state->lifetime.destroy();
 
 		const auto balance = session->credits().balance();
-		if (StarsAmount(credits) <= balance) {
+		if (CreditsAmount(credits) <= balance) {
 			if (const auto onstack = done) {
 				onstack(SmallBalanceResult::Already);
 			}
