@@ -911,7 +911,7 @@ void SetupTagContent(
 
 	const auto send = [=, weak = Ui::MakeWeak(tagsButton)](bool checked) {
 		session->data().chatsFilters().requestToggleTags(checked, [=] {
-			if (const auto strong = weak.data()) {
+			if ([[maybe_unused]] const auto strong = weak.data()) {
 				state->tagsTurnOff.fire(!checked);
 			}
 		});
