@@ -119,7 +119,7 @@ void InfoBox(
 				data.bot,
 				st::websiteBigUserpic)),
 		st::sessionBigCoverPadding)->entity();
-	userpic->forceForumShape(true);
+	userpic->overrideShape(Ui::PeerUserpicShape::Forum);
 	userpic->setAttribute(Qt::WA_TransparentForMouseEvents);
 
 	const auto nameWrap = box->addRow(
@@ -224,25 +224,11 @@ PaintRoundImageCallback Row::generatePaintUserpicCallback(bool forceRound) {
 	const auto peer = _data.bot;
 	auto userpic = _userpic = peer->createUserpicView();
 	return [=](Painter &p, int x, int y, int outerWidth, int size) mutable {
-		const auto ratio = style::DevicePixelRatio();
-		if (const auto cloud = peer->userpicCloudImage(userpic)) {
-			Ui::ValidateUserpicCache(
-				userpic,
-				cloud,
-				nullptr,
-				size * ratio,
-				true);
-			p.drawImage(QRect(x, y, size, size), userpic.cached);
-		} else {
-			if (_emptyUserpic.isNull()) {
-				_emptyUserpic = PeerData::GenerateUserpicImage(
-					peer,
-					_userpic,
-					size * ratio,
-					size * ratio * Ui::ForumUserpicRadiusMultiplier());
-			}
-			p.drawImage(QRect(x, y, size, size), _emptyUserpic);
-		}
+		peer->paintUserpic(p, _userpic, {
+			.position = QPoint(x, y),
+			.size = size,
+			.shape = Ui::PeerUserpicShape::Forum,
+		});
 	};
 }
 
