@@ -598,7 +598,24 @@ void ElasticScroll::touchResetSpeed() {
 }
 
 bool ElasticScroll::eventHook(QEvent *e) {
-	return filterOutTouchEvent(e) || RpWidget::eventHook(e);
+	if (filterOutTouchEvent(e)) {
+		if (e->type() == QEvent::TouchCancel
+			&& !_touchDisabled
+			&& _widget
+			&& _widget->testAttribute(Qt::WA_AcceptTouchEvents)) {
+			// If touch was cancelled for me, send the cancel
+			// event to the widget as well.
+			//
+			// In case scroll owner handled touch himself using
+			// _customTouchProcess hook and wants to cancel any
+			// actions here, we want to cancel them in widget too.
+			QTouchEvent ev(QEvent::TouchCancel);
+			ev.setTimestamp(crl::now());
+			QGuiApplication::sendEvent(_widget, &ev);
+		}
+		return true;
+	}
+	return RpWidget::eventHook(e);
 }
 
 void ElasticScroll::wheelEvent(QWheelEvent *e) {
@@ -662,8 +679,10 @@ bool ElasticScroll::handleWheelEvent(not_null<QWheelEvent*> e, bool touch) {
 			(unmultiplied.x() * std::max(width(), 120) / 120.),
 			(unmultiplied.y() * std::max(height(), 120) / 120.))
 		: unmultiplied;
+	auto ignore = false;
 	auto delta = _vertical ? -pixels.y() : pixels.x();
 	if (std::abs(_vertical ? pixels.x() : pixels.y()) >= std::abs(delta)) {
+		ignore = true;
 		delta = 0;
 	}
 	if (_ignoreMomentumFromOverscroll) {
@@ -699,7 +718,7 @@ bool ElasticScroll::handleWheelEvent(not_null<QWheelEvent*> e, bool touch) {
 		applyScrollTo(normalTo);
 	}
 	if (!delta) {
-		return true;
+		return !ignore;
 	}
 	if (touch) {
 		delta *= kTouchOverscrollMultiplier;
@@ -785,12 +804,14 @@ void ElasticScroll::handleTouchEvent(QTouchEvent *e) {
 		_touchPress = true;
 		if (_touchScrollState == TouchScrollState::Auto) {
 			_touchScrollState = TouchScrollState::Acceleration;
+			_touchMaybePressing = false;
 			_touchWaitingAcceleration = true;
 			_touchAccelerationTime = crl::now();
 			touchUpdateSpeed();
 			_touchStart = _touchPosition;
 		} else {
 			_touchScroll = false;
+			_touchMaybePressing = true;
 			_touchTimer.callOnce(QApplication::startDragTime());
 		}
 		_touchStart = _touchPreviousPosition = _touchPosition;
@@ -807,6 +828,7 @@ void ElasticScroll::handleTouchEvent(QTouchEvent *e) {
 				>= QApplication::startDragDistance())) {
 			_touchTimer.cancel();
 			_touchScroll = true;
+			_touchMaybePressing = false;
 			touchUpdateSpeed();
 		}
 		if (_touchScroll) {
@@ -861,12 +883,14 @@ void ElasticScroll::handleTouchEvent(QTouchEvent *e) {
 		if (weak) {
 			_touchTimer.cancel();
 			_touchRightButton = false;
+			_touchMaybePressing = false;
 		}
 	} break;
 
 	case QEvent::TouchCancel: {
 		_touchPress = false;
 		_touchScroll = false;
+		_touchMaybePressing = false;
 		_touchScrollState = TouchScrollState::Manual;
 		_touchTimer.cancel();
 	} break;
@@ -1290,6 +1314,10 @@ rpl::producer<ElasticScrollMovement> ElasticScroll::movementValue() const {
 	return _movement.value();
 }
 
+rpl::producer<bool> ElasticScroll::touchMaybePressing() const {
+	return _touchMaybePressing.value();
+}
+
 int OverscrollFromAccumulated(int accumulated) {
 	if (!accumulated) {
 		return 0;
